#include "modulemessage.h"

#include <QDateTime>

ModuleMessage::ModuleMessage()
{
    setTimestamp(QTime::currentTime());
}

ModuleMessage::ModuleMessage(const QString &topic) : ModuleMessage()
{
    setTopic(topic);
}

ModuleMessage::ModuleMessage(const QString &topic, const QString &msg) : ModuleMessage(topic)
{
    setPayload(msg);
}

ModuleMessage::ModuleMessage(const QString &topic, const QString &payload, const QTime &time) : ModuleMessage(topic, payload)
{
    setTimestamp(time);
}

ModuleMessage::ModuleMessage(const ModuleMessage &other)
{
    copy(other);
}

ModuleMessage ModuleMessage::operator=(const ModuleMessage &other)
{
    copy(other);
    return *this;
}

QString ModuleMessage::payload() const
{
    return _payload;
}

void ModuleMessage::setPayload(const QString &value)
{
    _payload = value;
}

void ModuleMessage::setPayload(int value)
{
    setPayload(QString::number(value));
}

QString ModuleMessage::topic() const
{
    return _topic;
}

void ModuleMessage::setTopic(const QString &value)
{
    _topic = value;
}

QTime ModuleMessage::timestamp() const
{
    return _timestamp;
}

void ModuleMessage::setTimestamp(const QTime &timestamp)
{
    _timestamp = timestamp;
}

QString ModuleMessage::getHigherHierarchyTopic() const
{
    int separatoIndex = _topic.lastIndexOf(_separator);
    if(separatoIndex < 0){
        separatoIndex = 0;
    }
    int len = _topic.length() - separatoIndex;
    return QString(_topic).remove(separatoIndex,len);
}

QString ModuleMessage::originalTopic() const
{
    return _originalTopic;
}

void ModuleMessage::setOriginalTopic(const QString &originalTopic)
{
    _originalTopic = originalTopic;
}

XmlObject ModuleMessage::getOptions() const
{
    return options;
}

QString ModuleMessage::getOption(const QString &optionName) const
{
    return options.getAttribute(optionName);
}

bool ModuleMessage::getIntOption(const QString &optionName, int &val) const
{
    return options.getIntAttribute(optionName, val);
}

void ModuleMessage::setOptions(const XmlObject &value)
{
    options = value;
}

void ModuleMessage::addOption(const QString &optionName, const QString &value)
{
    options.addAttribute(optionName, value);
}

void ModuleMessage::addOption(const QString &optionName, int value)
{
    options.addAttribute(optionName, value);
}

bool ModuleMessage::getIntPayload(int &value) const
{
    bool ok;
    if(payload() != ""){
        int temp = payload().toInt(&ok);
        if(ok){
            value = temp;
            return true;
        }
    }
    return false;
}

bool ModuleMessage::getFloatPayload(float &value) const
{
    bool ok;
    if(payload() != ""){
        int temp = payload().toFloat(&ok);
        if(ok){
            value = temp;
            return true;
        }
    }
    return false;
}

QString ModuleMessage::toString() const
{
    QString line = QDateTime::currentDateTime().toString("hh.mm.ss (zzz): ") + "(Topic) " + this->originalTopic() + " -> " + this->payload() + " (payload)";
    auto i = this->getOptions().attributesIterator();
    while(i.hasNext()){
        i.next();
        line = line + ", " + i.value() + " ("+i.key()+")";
    }
    return line;
}

void ModuleMessage::copy(const ModuleMessage &msg)
{
    setPayload(msg.payload());
    setTopic(msg.topic());
    setOriginalTopic(msg.originalTopic());
    setTimestamp(msg.timestamp());
    setOptions(msg.getOptions());
}
