#ifndef MODULEMESSAGE_H
#define MODULEMESSAGE_H


#include <QString>
#include <QDateTime>
#include "xmlobject.h"

class ModuleMessage
{
public:
    ModuleMessage();
    ModuleMessage(const QString &topic);
    ModuleMessage(const QString &_topic, const QString &payload);
    ModuleMessage(const QString &_topic, const QString &payload, const QTime &time);
    ModuleMessage(const ModuleMessage &other);

    ModuleMessage operator=(const ModuleMessage &other);

    QString payload() const;
    void setPayload(const QString &value);
    void setPayload(int value);

    QString topic() const;
    void setTopic(const QString &value);

    QTime timestamp() const;
    void setTimestamp(const QTime &timestamp);

    QString getHigherHierarchyTopic() const;

    QString originalTopic() const;
    void setOriginalTopic(const QString &originalTopic);

    /*
     * Returns an XmlObject containing some options
     */
    XmlObject getOptions() const;
    QString getOption(const QString &optionName) const;
    bool getIntOption(const QString &optionName, int &val) const;
    void setOptions(const XmlObject &value);
    void addOption(const QString &optionName, const QString &value);
    void addOption(const QString &optionName, int value);

    /*
     * Get the value of "value" if the payload exist and is an integer
     */
    bool getIntPayload(int &value) const;

    bool getFloatPayload(float &value) const;

    QString toString() const;

private:
    void copy(const ModuleMessage &msg);

    QString _payload;
    QString _topic;
    QString _originalTopic;
    QTime _timestamp;
    const QChar _separator = '/';
    XmlObject options;
};

#endif // MODULEMESSAGE_H
