#include "modulemessagesbroker.h"

#include <QDebug>
#include "module.h"

//ModuleMessagesBroker *ModuleMessagesBroker::modulesMsgBroker = nullptr;

ModuleMessagesBroker::ModuleMessagesBroker()
{
    qDebug() << "ModulesManager >> Created";
}

ModuleMessagesBroker::~ModuleMessagesBroker()
{

}

void ModuleMessagesBroker::subscribe(const QString &topic, Module *observer, std::function<void(const ModuleMessage &msg)> callBack)
{
    observers[topic][observer].append(callBack);
    connect(observer->getModuleEventsHandler(),&ModuleEventsHandler::beforeDelete, this, &ModuleMessagesBroker::onModuleDeleted);
}

void ModuleMessagesBroker::unsubscribe(const QString &topic, Module *module)
{
    if(observers.contains(topic)){
        if(observers[topic].contains(module)){
            observers[topic].remove(module);
            if(observers[topic].isEmpty()){
                observers.remove(topic);
            }
        }
    }
}

void ModuleMessagesBroker::publish(const QString &topic, const QString &msg)
{
    ModuleMessage m(topic,msg);
    publish(m);
}

void ModuleMessagesBroker::publish(const ModuleMessage &msg)
{
    ModuleMessage m = msg;
    m.setOriginalTopic(msg.topic());
    while(m.topic().length() > 0){
        send(m);
        m.setTopic(m.getHigherHierarchyTopic());
    }
}


void ModuleMessagesBroker::send(const ModuleMessage &msg)
{
    if(observers.contains(msg.topic())){
        for(auto calls : observers[msg.topic()].values()){
            for(auto callBack : calls){
                callBack(msg);
            }
        }
    }
}

QList<QString> ModuleMessagesBroker::getSubscriptionsOf(Module *module)
{
    QList<QString> topics;

    for(auto topic : observers.keys()){
        auto observer  = observers[topic];
        if(observer.contains(module)){
            topics.append(topic);
        }
    }
    return topics;
}

void ModuleMessagesBroker::onModuleDeleted(Module *module)
{
    for(auto topic : observers.keys()){
        if(observers[topic].contains(module)){
            observers[topic].remove(module);
        }
        if(observers[topic].isEmpty()){
            observers.remove(topic);
        }
    }
}



//_________________ ModulesManagerProxy ____________________________________________________________

//ModuleMessagesBroker *ModuleMessagesBrokersProxy::getModuleMessagesBroker()
//{
//    if(ModuleMessagesBrokersProxy::modulesMsgBroker == nullptr){
//        ModuleMessagesBrokersProxy::modulesMsgBroker = new ModuleMessagesBroker();
//    }
//    return ModuleMessagesBrokersProxy::modulesMsgBroker;
//}

//_________________ ModulesManagerProxy ____________________________________________________________
