#ifndef MODULESMANAGER_H
#define MODULESMANAGER_H

#include <QObject>
#include <QString>
#include <QMap>
#include <QMenu>
#include "Modules/moduleslist.h"

class Module;
class SaveConfigurationDialog;
class XmlObject;

class ModulesManager : public QObject
{
    Q_OBJECT

public:
    ModulesManager();
    ~ModulesManager();

    static Module* instantiateDefaultModule();
    static Module* invokeModulesPickerService();

    static QList<QString> getModulesNamesList();
    static QString getModuleName(ModuleId id);
    static Module* instantiateModuleById(ModuleId id);
    static Module* instantiateModuleById(ModuleId id, const XmlObject &params);

    static Module* instantiateModuleByName(const QString &moduleName);
    static Module* instantiateModuleByName(const QString &moduleName, const XmlObject &params);
    static QList<Module *> loadModuleFromXml(XmlObject &xml);
    static QList<ModuleInfo> getModulesInfo();



    void setModules(QList<Module*> modules);
    void addModules(QList<Module*> modules);
    void addPage(Module *module);
    void removePage(Module *module);
    void openNewEmptyWindow();

    Module* getModuleAt(int index);
    int getModuleCount();
    void clearPages();


    bool getRebuild() const;
    void setRebuild(bool value);


    QList<QAction *> getHubMenuActions();

signals:
    void contextMenuRequest(QMenu &menu,const QPoint &p);
    void pageAdded(int index);
    void appQuitRequested();

protected:
    void connectModule(Module *module);
    void disconnectModule(Module *module);
    void saveConfigurationService();
    void onFirstPageDeleted();

protected slots:
    void onContextMenuRequest(QMenu &menu, const QPoint &p);
    void onModuleDeleted(Module *deletedModule);
    void onReplaceMeWith(Module *sender, Module *newModule);

private:
    static ModulesList modulesListHandler;

    QList<Module*> pages;
    QList<QAction*> menuActions;
    bool rebuild =  true;
};

#endif // MODULESMANAGER_H
