#include "skywardhubcore.h"

#include <QDir>
#include <QMessageBox>

#include "modulesmanager.h"
#include "moduleeventshandler.h"
#include "modulemessagesbroker.h"

#include "module.h"

SkywardHubCore::SkywardHubCore()
{
    modulesManager = new ModulesManager();
    moduleEventsHandler = new ModuleEventsHandler();
    moduleMessagesBroker = new ModuleMessagesBroker();
}

SkywardHubCore::~SkywardHubCore()
{
    if(modulesManager){
        delete modulesManager;
    }
    if(moduleEventsHandler){
        delete moduleEventsHandler;
    }
    if(moduleMessagesBroker){
        delete moduleMessagesBroker;
    }
}

void SkywardHubCore::init()
{
    if(!settings.loadFromFile(SkywardHubStrings::defaultSettingsFilePath)){
        // Settings file not found or loaded incorrectly
        settings.reset();
        settings.setObjectName(SkywardHubStrings::settingsObjectName);
        XmlObject defaultConfig(SkywardHubStrings::skywardHubInitFileTag);
        defaultConfig.setTextValue(SkywardHubStrings::defaultConfigurationFileName);
        settings.addChild(defaultConfig);
    }

    checkDefaultFolders();
    loadFirstPage();
}

void SkywardHubCore::checkDefaultFolders()
{
    /*
     * If SkywardHubConfig is missing, it create all the missing folder
     */

    if(!QDir(SkywardHubStrings::defaultConfigurationFolder).exists()){
        QDir().mkdir(SkywardHubStrings::defaultConfigurationFolder);
    }

    if(!QFile(SkywardHubStrings::defaultSettingsFilePath).exists()){
        saveSettings();
    }
}

void SkywardHubCore::saveSettings()
{
    if(settings.writeToFile(SkywardHubStrings::defaultSettingsFilePath)){
        QMessageBox confirmMsg;
        confirmMsg.setMinimumSize(256, 256);
        confirmMsg.setText(SkywardHubStrings::settingsSavedCorrectlyMsg);
        confirmMsg.exec();
    }
}

void SkywardHubCore::loadFirstPage()
{
    QString pageToLoadName = settings.getChildObjectValue(SkywardHubStrings::skywardHubInitFileTag);
    if(!pageToLoadName.isEmpty()){
        XmlObject pageToLoad;
        if(pageToLoad.loadFromFile(SkywardHubStrings::defaultPrefabsFolder + pageToLoadName)){
            modulesManager->setModules(modulesManager->loadModuleFromXml(pageToLoad));
            return;
        }
    }

    // if first page not found, init with empy module
    modulesManager->openNewEmptyWindow();
}

ModuleMessagesBroker *SkywardHubCore::getModuleMessagesBroker()
{
    return moduleMessagesBroker;
}

ModuleEventsHandler *SkywardHubCore::getModuleEventsHandler()
{
    return moduleEventsHandler;
}

ModulesManager *SkywardHubCore::getModulesManager()
{
    return modulesManager;
}

//_________________ SkywardHubCoreProxy ____________________________________________________________

SkywardHubCore *SkywardHubCoreProxy::core = nullptr;

SkywardHubCore *SkywardHubCoreProxy::getCore()
{
    if(SkywardHubCoreProxy::core == nullptr){
        SkywardHubCoreProxy::core = new SkywardHubCore();
    }
    return SkywardHubCoreProxy::core;
}

//_________________ ModulesManagerProxy ____________________________________________________________

XmlObject SkywardHubCore::getSettings() const
{
    return settings;
}
