#ifndef XMLOBJECT_H
#define XMLOBJECT_H


#include <QMap>
#include <QXmlStreamWriter>
#include <QXmlStreamReader>
#include <QMapIterator>

class XmlObject
{
public:
    XmlObject(const QString &objectName = "Object");
    XmlObject(const XmlObject &other);


    bool loadFromFile(const QString& filePath);
    bool writeToFile(const QString& filePath, const QString &codec = "UTF-8") const;

    QString toXml() const;
    void fromXml(const QString &txt);

    QString getAttribute(const QString &name) const;
    bool hasAttribute(const QString &name) const;
    int attributeCount() const;
    QMapIterator<QString, QString> attributesIterator() const;

    /*
     * Get the value of "value" if the attribute with name "name" exist and is an integer
     */
    bool getIntAttribute(const QString &name, int &value) const;
    bool getFloatAttribute(const QString &name, float &value) const;
    /*
     * addAttribute create a new attribute with name "name" and value "value"
     * If the attribute already exist, it change its value
     */
    void addAttribute(const QString &name, const QString &value);
    void addAttribute(const QString &name, int value);
    int addChild(const XmlObject &c);
    void addChilds(const QList<XmlObject> &childs);

    QString getObjectName() const;
    void setObjectName(const QString &value);

    int childCount() const;
    XmlObject childAt(int index) const;
    XmlObject searchChild(const QString &name) const;
    /*
     * getChild Returns the child at index position as a modifiable reference.
     * index must be a valid index position in the list
     */
    XmlObject *getChild(int index);

    /*
     * Is the text inside the xml tag
     * es: <tag> text value </tag>
     */
    QString getTextValue() const;
    void setTextValue(const QString &value);

    /*
     * deepSearchObjects return all the child objects that satisfy the predicate
     */
    QList<XmlObject*> deepSearchObjects(std::function<bool(const XmlObject *obj)> predicate);
    QList<XmlObject*> deepSearchObjects(const QString &objectName);

    void operator = (const XmlObject &other );
    void removeChild(int i);

    /*
     * Exchange the child at index position i with the item at index position j.
     * This function assumes that both i and j are at least 0 but less than
     */
    void swapChildAt(int i, int j);

    QString getChildObjectValue(const QString &objectName);

    void clearAttributes();
    void clearChilds();
    void reset();

    bool isEmpty();
protected:
    void toXml(QXmlStreamWriter *xmlWriter) const;
    void fromXml(QXmlStreamReader *xmlReader);
    void copy(const XmlObject &other);

private:
    QMap<QString,QString> attributes;
    QList<XmlObject> childList;
    QString textValue = "";
    QString xmlObjectName = "";

};

#endif // XMLOBJECT_H
