#include "commandpadmodule.h"
#include "ui_commandpadmodule.h"

#include "Core/modulesmanager.h"
#include "Core/modulemessagesbroker.h"
#include "Modules/moduleslist.h"
#include "Modules/skywardhubstrings.h"
#include <QDebug>

CommandPadModule::CommandPadModule(QWidget *parent) : DefaultModule(parent), ui(new Ui::CommandPadModule)
{
    ui->setupUi(this);
    defaultContextMenuSetup();
    buildUI();
}

CommandPadModule::~CommandPadModule()
{
    delete ui;
}

QWidget *CommandPadModule::toWidget()
{
    return this;
}

XmlObject CommandPadModule::toXmlObject()
{
    return XmlObject(getName(ModuleId::COMMANDPAD));
}

void CommandPadModule::fromXmlObject(const XmlObject &xmlObject)
{
    Q_UNUSED(xmlObject);
}

void CommandPadModule::buildUI()
{
    connect(ui->closeLog_button, &QPushButton::clicked, this, &CommandPadModule::onCloseLogClicked);
    connect(ui->wiggleARBServo_button, &QPushButton::clicked, this, &CommandPadModule::onWiggleARBServoClicked);
    connect(ui->resetARBServo_button, &QPushButton::clicked, this, &CommandPadModule::onResetARBServoClicked);
    connect(ui->wiggleDPLServo_button, &QPushButton::clicked, this, &CommandPadModule::onWiggleDPLServoClicked);
    connect(ui->resetDPLServo_button, &QPushButton::clicked, this, &CommandPadModule::onResetDPLServoClicked);
//    connect(ui->testPrimaryCut_button, &QPushButton::clicked, this, &CommandPadModule::onTestPrimaryCutClicked);
    connect(ui->testMode_button, &QPushButton::clicked, this, &CommandPadModule::onTestModeClicked);
//    connect(ui->testBackupCut_button, &QPushButton::clicked, this, &CommandPadModule::onTestBackupCutClicked);
    connect(ui->boardReset_button, &QPushButton::clicked, this, &CommandPadModule::onBoardResetClicked);
    connect(ui->testAerobrakest_button, &QPushButton::clicked, this, &CommandPadModule::onTestAerobrakesClicked);
    connect(ui->disableAerobrakes_button, &QPushButton::clicked, this, &CommandPadModule::onDisableAerobrakesClicked);
    connect(ui->deployAltitudeSet_button, &QPushButton::clicked, this, &CommandPadModule::onDeploymentAltitudeSetClicked);
    connect(ui->tempeatureSet_button, &QPushButton::clicked, this, &CommandPadModule::onRefTemperatureSetClicked);
    connect(ui->initialCoordinatesSet_button, &QPushButton::clicked, this, &CommandPadModule::onInitialCoordinatesSetClicked);
    connect(ui->referenceAltitudeSet_button, &QPushButton::clicked, this, &CommandPadModule::onRefAltitudeSetClicked);
    connect(ui->aerobrakeAngleSet_button, &QPushButton::clicked, this, &CommandPadModule::onAerobrakeAngleSetClicked);
    connect(ui->initialOrientationSet_pushButton, &QPushButton::clicked, this, &CommandPadModule::onInitialOrientationSetClicked);
    connect(ui->arm_button, &QPushButton::clicked, this, &CommandPadModule::onArmClicked);
    connect(ui->disarm_button, &QPushButton::clicked, this, &CommandPadModule::onDisarmClicked);
    connect(ui->forceLaunch_button, &QPushButton::clicked, this, &CommandPadModule::onForceLaunchClicked);
    connect(ui->forceInit_button, &QPushButton::clicked, this, &CommandPadModule::onForceInitClicked);
    connect(ui->noseconeOpen_button, &QPushButton::clicked, this, &CommandPadModule::onNoseconeOpenClicked);
    connect(ui->endMission_button, &QPushButton::clicked, this, &CommandPadModule::onEndMissionClicked);
    connect(ui->rawEventSend_button, &QPushButton::clicked, this, &CommandPadModule::onRawEventSendClicked);
    connect(ui->calibrateAlgos_button, &QPushButton::clicked, this, &CommandPadModule::onCalibrateALGOSClicked);
    connect(ui->calibrateSensors_button, &QPushButton::clicked, this, &CommandPadModule::onCalibrateSENSORSClicked);
    connect(ui->startLog_button, &QPushButton::clicked, this, &CommandPadModule::onStartLogClicked);
    connect(ui->startTelemetry_button, &QPushButton::clicked, this, &CommandPadModule::onStartTelemetryClicked);
    connect(ui->stopTelemetry_button, &QPushButton::clicked, this, &CommandPadModule::onStopTelemetryClicked);

    connect(ui->cutDrogue_button, &QPushButton::clicked, this, &CommandPadModule::onCutDrogueClicked);
//    connect(ui->primaryCut_button, &QPushButton::clicked, this, &CommandPadModule::onPrimaryCutClicked);
//    connect(ui->backupCut_button, &QPushButton::clicked, this, &CommandPadModule::onBackupClicked);
}

//void CommandPadModule::buildMsgKeyList()
//{
//    msgKeyList = QList<QString>({

//        "MAV_CMD_ARM",
//        "MAV_CMD_DISARM",
//        "MAV_CMD_FORCE_LAUNCH",
//        "MAV_CMD_CALIBRATE_ADA",
//        "MAV_CMD_FORCE_INIT",
//        "MAV_CMD_NOSECONE_OPEN",
//        "MAV_CMD_RESET_SERVO",
//        "MAV_CMD_WIGGLE_SERVO",
//        "MAV_CMD_CUT_DROGUE",
//        "MAV_CMD_CUT_PRIMARY",
//        "MAV_CMD_CUT_BACKUP",
//        "MAV_CMD_TEST_PRIMARY_CUTTER",
//        "MAV_CMD_TEST_BACKUP_CUTTER",
//        "MAV_CMD_START_LOGGING",
//        "MAV_CMD_STOP_LOGGING",
//        "MAV_CMD_CLOSE_LOG",
//        "MAV_CMD_END_MISSION",
//        "MAV_CMD_BOARD_RESET",
//        "MAV_CMD_TEST_MODE",

//        "MAV_SET_DEPLOYMENT_ALTITUDE",
//        "MAV_SET_REFERENCE_TEMP",
//        "MAV_SET_REFERENCE_ALTITUDE"

//    });
//}

ModuleMessage CommandPadModule::createCommandMsg(const QString &msgKey) const
{
    ModuleMessage msg;
    msg.setTopic(SkywardHubStrings::commandsTopic + "/" + msgKey);
    return msg;
}

ModuleMessage CommandPadModule::createCommandMsgWithPayload(const QString &msgKey, const QString &payload) const
{
    ModuleMessage msg = createCommandMsg(msgKey);
    msg.setPayload(payload);
    return msg;
}


ModuleMessage CommandPadModule::createRawEventMsg(const QString &msgKey, int event_id, int topic_id) const
{
    ModuleMessage msg = createCommandMsg(msgKey);

    msg.addOption(SkywardHubStrings::raw_event_id, event_id);
    msg.addOption(SkywardHubStrings::raw_event_topic_id, topic_id);

    return msg;
}

void CommandPadModule::send(const ModuleMessage &msg)
{
    getCore()->getModuleMessagesBroker()->publish(msg);
}


void CommandPadModule::onStartLogClicked()
{
    send(createCommandMsg("MAV_CMD_START_LOGGING"));
//    if(val){
//        // Start logging
//        send(createCommandMsg("MAV_CMD_START_LOGGING"));
//    }
//    else{
//        // Stop logging
//        send(createCommandMsg("MAV_CMD_STOP_LOGGING"));
//    }
}

void CommandPadModule::onCloseLogClicked()
{
    send(createCommandMsg("MAV_CMD_CLOSE_LOG"));
}

void CommandPadModule::onWiggleARBServoClicked()
{
    send(createCommandMsg("MAV_CMD_ARB_WIGGLE_SERVO"));
}

void CommandPadModule::onResetARBServoClicked()
{
    send(createCommandMsg("MAV_CMD_ARB_RESET_SERVO"));
}

void CommandPadModule::onWiggleDPLServoClicked()
{
    send(createCommandMsg("MAV_CMD_DPL_WIGGLE_SERVO"));
}

void CommandPadModule::onResetDPLServoClicked()
{
    send(createCommandMsg("MAV_CMD_DPL_RESET_SERVO"));
}

//void CommandPadModule::onTestPrimaryCutClicked()
//{
//    send(createCommandMsg("MAV_CMD_TEST_PRIMARY_CUTTER"));
//}

//void CommandPadModule::onTestBackupCutClicked()
//{
//    send(createCommandMsg("MAV_CMD_TEST_BACKUP_CUTTER"));
//}

void CommandPadModule::onTestModeClicked()
{
    send(createCommandMsg("MAV_CMD_TEST_MODE"));
}

void CommandPadModule::onBoardResetClicked()
{
    send(createCommandMsg("MAV_CMD_BOARD_RESET"));
}

void CommandPadModule::onCalibrateALGOSClicked()
{
    send(createCommandMsg("MAV_CMD_CALIBRATE_ALGOS"));
}

void CommandPadModule::onCalibrateSENSORSClicked()
{
    send(createCommandMsg("MAV_CMD_CALIBRATE_SENSORS"));
}

void CommandPadModule::onTestAerobrakesClicked()
{
    send(createCommandMsg("MAV_CMD_TEST_AEROBRAKES"));
}

void CommandPadModule::onDisableAerobrakesClicked()
{
    send(createCommandMsg("MAV_CMD_DISABLE_AEROBRAKES"));
}

void CommandPadModule::onDeploymentAltitudeSetClicked()
{
    QString payload = QString::number(ui->deploymentAltitude_spinBox->value());
    send(createCommandMsgWithPayload("SET_DEPLOYMENT_ALTITUDE_TC", payload));
}

void CommandPadModule::onRefTemperatureSetClicked()
{
    QString payload = QString::number(ui->refTemperature_spinBox->value());
    send(createCommandMsgWithPayload("SET_REFERENCE_TEMPERATURE_TC", payload));
}

void CommandPadModule::onRefAltitudeSetClicked()
{   
    QString payload = QString::number(ui->referenceAltitude_spinBox->value());
    send(createCommandMsgWithPayload("SET_REFERENCE_ALTITUDE", payload));
}

void CommandPadModule::onInitialCoordinatesSetClicked()
{
    ModuleMessage msg = createCommandMsg("SET_INITIAL_COORDINATES_TC");

    QString lat = QString::number(ui->initialLatitude_spinBox->value());
    QString longit = QString::number(ui->initialLongitude_spinBox->value());

    msg.addOption(SkywardHubStrings::mavlink_initial_coordinates_tc_latitude_name, lat);
    msg.addOption(SkywardHubStrings::mavlink_initial_coordinates_tc_longitude_name, longit);

    send(msg);
}

void CommandPadModule::onAerobrakeAngleSetClicked()
{
    QString payload = QString::number(ui->aerobrakeAngle_spinBox->value());
    send(createCommandMsgWithPayload("SET_AEROBRAKE_ANGLE_TC", payload));
}

void CommandPadModule::onInitialOrientationSetClicked()
{    
    ModuleMessage msg = createCommandMsg("SET_INITIAL_ORIENTATION_TC");

    QString yaw = QString::number(ui->initialOrientationYaw_spinBox->value());
    QString pitch = QString::number(ui->initialOrientationPitch_spinBox->value());
    QString roll = QString::number(ui->initialOrientationRoll_spinBox->value());

    msg.addOption(SkywardHubStrings::mavlink_orientation_tc_yaw_name, yaw);
    msg.addOption(SkywardHubStrings::mavlink_orientation_tc_pitch_name, pitch);
    msg.addOption(SkywardHubStrings::mavlink_orientation_tc_roll_name, roll);

    send(msg);
}

void CommandPadModule::onArmClicked()
{
    send(createCommandMsg("MAV_CMD_ARM"));
}

void CommandPadModule::onDisarmClicked()
{
    send(createCommandMsg("MAV_CMD_DISARM"));
}

void CommandPadModule::onForceLaunchClicked()
{
    send(createCommandMsg("MAV_CMD_FORCE_LAUNCH"));
}

void CommandPadModule::onForceInitClicked()
{
    send(createCommandMsg("MAV_CMD_FORCE_INIT"));
}

void CommandPadModule::onNoseconeOpenClicked()
{
    send(createCommandMsg("MAV_CMD_NOSECONE_OPEN"));
}

void CommandPadModule::onEndMissionClicked()
{
    send(createCommandMsg("MAV_CMD_END_MISSION"));
}

void CommandPadModule::onRawEventSendClicked()
{
    int event_id = ui->rawEventId_spinBox->value();
    int topic_id = ui->rawEventTopic_spinBox->value();
    send(createRawEventMsg("RAW_EVENT_TC", event_id, topic_id));
}

void CommandPadModule::onCutDrogueClicked()
{
    send(createCommandMsg("MAV_CMD_CUT_DROGUE"));
}

//void CommandPadModule::onPrimaryCutClicked()
//{
//    send(createCommandMsg("MAV_CMD_CUT_PRIMARY"));
//}

//void CommandPadModule::onBackupClicked()
//{
//    send(createCommandMsg("MAV_CMD_CUT_BACKUP"));
//}

void CommandPadModule::onStartTelemetryClicked()
{
    send(createCommandMsg("MAV_CMD_RADIO_TM"));
}

void CommandPadModule::onStopTelemetryClicked()
{
    send(createCommandMsg("MAV_CMD_SERIAL_TM"));
}

