#include "telemetryrequestmodule.h"
#include "ui_telemetryrequestmodule.h"

#include "Core/modulemessagesbroker.h"
#include "Modules/skywardhubstrings.h"

TelemetryRequestModule::TelemetryRequestModule(QWidget *parent) : DefaultModule(parent), ui(new Ui::TelemetryRequestModule)
{
    ui->setupUi(this);
    defaultContextMenuSetup();
    connectUI();
}

TelemetryRequestModule::~TelemetryRequestModule()
{
    delete ui;
}

QWidget *TelemetryRequestModule::toWidget()
{
    return this;
}

XmlObject TelemetryRequestModule::toXmlObject()
{
    return XmlObject(getName(ModuleId::TELEMETRYPAD));
}

void TelemetryRequestModule::fromXmlObject(const XmlObject &xmlObject)
{
    Q_UNUSED(xmlObject);
}

void TelemetryRequestModule::connectUI()
{
    connect(ui->sys_tm_pushButton, &QPushButton::clicked, this, &TelemetryRequestModule::on_SYS_TM_Clicked);
    connect(ui->fmm_tm_pushButton, &QPushButton::clicked, this, &TelemetryRequestModule::on_FMM_TM_Clicked);
    connect(ui->logger_tm_pushButton, &QPushButton::clicked, this, &TelemetryRequestModule::on_LOGGER_TM_Clicked);
    connect(ui->tmtc_tm_pushButton, &QPushButton::clicked, this, &TelemetryRequestModule::on_TMTC_TM_Clicked);
    connect(ui->dpl_tm_pushButton, &QPushButton::clicked, this, &TelemetryRequestModule::on_DPL_TM_Clicked);
    connect(ui->ada_tm_pushButton, &QPushButton::clicked, this, &TelemetryRequestModule::on_ADA_TM_Clicked);
//    connect(ui->can_tm_pushButton, &QPushButton::clicked, this, &TelemetryRequestModule::on_CAN_TM_Clicked);
    connect(ui->adc_tm_pushButton, &QPushButton::clicked, this, &TelemetryRequestModule::on_ADC_TM_Clicked);
    connect(ui->gps_tm_pushButton, &QPushButton::clicked, this, &TelemetryRequestModule::on_GPS_TM_Clicked);
    connect(ui->hr_tm_pushButton, &QPushButton::clicked, this, &TelemetryRequestModule::on_HR_TM_Clicked);
    connect(ui->lr_tm_pushButton, &QPushButton::clicked, this, &TelemetryRequestModule::on_LR_TM_Clicked);
    connect(ui->test_tm_pushButton, &QPushButton::clicked, this, &TelemetryRequestModule::on_TEST_TM_Clicked);

    connect(ui->pin_obs_tm_pushButton, &QPushButton::clicked, this, &TelemetryRequestModule::on_PIN_OBS_TM_Clicked);
    connect(ui->task_stats_tm_pushButton, &QPushButton::clicked, this, &TelemetryRequestModule::on_TASK_STATS_TM_Clicked);
    connect(ui->abk_tm_pushButton, &QPushButton::clicked, this, &TelemetryRequestModule::on_ABK_TM_Clicked);
    connect(ui->nas_tm_pushButton, &QPushButton::clicked, this, &TelemetryRequestModule::on_NAS_TM_Clicked);
    connect(ui->ms5803_tm_pushButton, &QPushButton::clicked, this, &TelemetryRequestModule::on_MS5803_TM_Clicked);
    connect(ui->bmx160_tm_pushButton, &QPushButton::clicked, this, &TelemetryRequestModule::on_BMX160_TM_Clicked);
    connect(ui->lis3mdl_tm_pushButton, &QPushButton::clicked, this, &TelemetryRequestModule::on_LIS3MDL_TM_Clicked);
//    connect(ui->strain_board_tm_pushButton, &QPushButton::clicked, this, &TelemetryRequestModule::on_STRAIN_BOARD_TM_Clicked);

    connect(ui->windtunnel_tm_pushButton, &QPushButton::clicked, this, &TelemetryRequestModule::on_WINDTUNNEL_TM_Clicked);
    connect(ui->sensors_tm_pushButton, &QPushButton::clicked, this, &TelemetryRequestModule::on_SENSORS_TM_Clicked);
}

void TelemetryRequestModule::send(const ModuleMessage &msg)
{
    getCore()->getModuleMessagesBroker()->publish(msg);
}

ModuleMessage TelemetryRequestModule::createTmMessage(const QString &msgKey)
{
    ModuleMessage msg;
    msg.setTopic(SkywardHubStrings::telemetryRequestTopic + "/" + msgKey);
    return msg;
}

void TelemetryRequestModule::on_SYS_TM_Clicked()
{
    send(createTmMessage("MAV_SYS_TM_ID"));
}

void TelemetryRequestModule::on_FMM_TM_Clicked()
{
    send(createTmMessage("MAV_FMM_TM_ID"));
}

void TelemetryRequestModule::on_LOGGER_TM_Clicked()
{
    send(createTmMessage("MAV_LOGGER_TM_ID"));
}

void TelemetryRequestModule::on_TMTC_TM_Clicked()
{
    send(createTmMessage("MAV_TMTC_TM_ID"));
}

void TelemetryRequestModule::on_DPL_TM_Clicked()
{
    send(createTmMessage("MAV_DPL_TM_ID"));
}

void TelemetryRequestModule::on_ADA_TM_Clicked()
{
    send(createTmMessage("MAV_ADA_TM_ID"));
}

//void TelemetryRequestModule::on_CAN_TM_Clicked()
//{
//    send(createTmMessage("MAV_CAN_TM_ID"));
//}

void TelemetryRequestModule::on_ADC_TM_Clicked()
{
    send(createTmMessage("MAV_ADC_TM_ID"));
}

void TelemetryRequestModule::on_GPS_TM_Clicked()
{
    send(createTmMessage("MAV_GPS_TM_ID"));
}

void TelemetryRequestModule::on_HR_TM_Clicked()
{
    send(createTmMessage("MAV_HR_TM_ID"));
}

void TelemetryRequestModule::on_LR_TM_Clicked()
{
    send(createTmMessage("MAV_LR_TM_ID"));
}

void TelemetryRequestModule::on_TEST_TM_Clicked()
{
    send(createTmMessage("MAV_TEST_TM_ID"));
}

void TelemetryRequestModule::on_WINDTUNNEL_TM_Clicked()
{
    send(createTmMessage("MAV_WINDTUNNEL_TM_ID"));
}

void TelemetryRequestModule::on_SENSORS_TM_Clicked()
{
    send(createTmMessage("MAV_SENSORS_TM_ID"));
}

void TelemetryRequestModule::on_PIN_OBS_TM_Clicked()
{
    send(createTmMessage("MAV_PIN_OBS_TM_ID"));
}

void TelemetryRequestModule::on_TASK_STATS_TM_Clicked()
{
    send(createTmMessage("MAV_TASK_STATS_TM_ID"));
}

void TelemetryRequestModule::on_ABK_TM_Clicked()
{
    send(createTmMessage("MAV_ABK_TM_ID"));
}

void TelemetryRequestModule::on_NAS_TM_Clicked()
{
    send(createTmMessage("MAV_NAS_TM_ID"));
}

void TelemetryRequestModule::on_MS5803_TM_Clicked()
{
    send(createTmMessage("MAV_MS5803_TM_ID"));
}

void TelemetryRequestModule::on_BMX160_TM_Clicked()
{
    send(createTmMessage("MAV_BMX160_TM_ID"));
}

void TelemetryRequestModule::on_LIS3MDL_TM_Clicked()
{
    send(createTmMessage("MAV_LIS3MDL_TM_ID"));
}

//void TelemetryRequestModule::on_STRAIN_BOARD_TM_Clicked()
//{
//    send(createTmMessage("MAV_STRAIN_BOARD_TM_ID"));
//}


