#include "defaultmodule.h"
#include <QCloseEvent>

#include "Modules/Utility/contextmenuseparator.h"
#include "Core/modulesmanager.h"

DefaultModule::DefaultModule(QWidget *parent) : QWidget(parent)
{

}

DefaultModule::~DefaultModule()
{
    for(int i = 0; i < menuActions.count(); i++){
        delete menuActions[i];
    }
    menuActions.clear();
}

void DefaultModule::closeEvent(QCloseEvent *event)
{
    if (event->spontaneous()) {
        emit getModuleEventsHandler()->beforeDelete(this);
        QWidget::closeEvent(event);
    } else {
        QWidget::closeEvent(event);
    }
}

void DefaultModule::defaultContextMenuSetup()
{
    if(areDefaultSettingsDone)
        return;

    areDefaultSettingsDone = true;
    setContextMenuPolicy(Qt::CustomContextMenu);
    connect(this,&DefaultModule::customContextMenuRequested, this, &DefaultModule::onCustomContextMenuRequested);
    addDefaultActionsToMenu();
    addCustomActionsToMenu();
}

void DefaultModule::onCustomContextMenuRequested(const QPoint &p)
{
    QMenu menu;
    onSkywardHubContextMenuRequested(menu, mapToGlobal(p));
}

QString DefaultModule::getName(const ModuleId id)
{
    return  getCore()->getModulesManager()->getModuleName(id);
}

void DefaultModule::onSkywardHubContextMenuRequested(QMenu &menu, const QPoint &p)
{
    QMenu newMenu(menuName);
    if(!menuName.isEmpty()){
        newMenu.addActions(menuActions);
        menu.addMenu(&newMenu);
    }
    else
        menu.addActions(menuActions);

    ContextMenuSeparator separator;
    menu.addAction(&separator);
    emit getModuleEventsHandler()->contextMenuRequest(menu, p);
}

void DefaultModule::addDefaultActionsToMenu()
{
    QAction* close = new QAction("Close");
    connect(close,&QAction::triggered,this,[this](){
        delete this;
    });

    QAction* replace = new QAction("Replace");
    connect(replace,&QAction::triggered,this, &DefaultModule::onReplaceClicked);

    addActionToMenu(close);
    addActionToMenu(replace);
}

void DefaultModule::addActionToMenu(QAction *action)
{
    menuActions.append(action);
}

void DefaultModule::showMenuActionsInSeparatedMenu(const QString &menuName)
{
    this->menuName = menuName;
}

void DefaultModule::onReplaceClicked()
{
    Module *newModule = getCore()->getModulesManager()->invokeModulesPickerService();
    if(newModule != nullptr){
        emit getModuleEventsHandler()->replaceMeWith(this, newModule);
        delete this;
    }
}

void DefaultModule::addCustomActionsToMenu()
{

}

SkywardHubCoreProxy &DefaultModule::getCore()
{
    return proxyCore;
}
