#ifndef GRAPHMODULE_H
#define GRAPHMODULE_H

#include <QWidget>
#include <QList>
#include "Core/module.h"
#include "Core/modulemessage.h"
#include "qcustomplot.h"
#include "Modules/DefaultModule/defaultmodule.h"
#include <QTimer>

namespace Ui {
class GraphModule;
}

class GraphModule : public DefaultModule
{
    Q_OBJECT

public:
    explicit GraphModule(QWidget *parent = nullptr);
    ~GraphModule();

    QWidget* toWidget() override;
    XmlObject toXmlObject() override;
    void fromXmlObject(const XmlObject &xmlObject) override;

    void centerView(const QCPGraph * graph);
    void replotAll();

public slots:
    void onSubscribeClicked();
    void onSubscriptionAdded(const QString &subscription);
    void onSubscriptionRemoved(const QString &subscription);
    void onMsgReceived(const ModuleMessage &msg);
    void setFollowedGraphIndex(bool checked);
    void onClearClicked();
    void onStopClicked(bool checked);

protected:
    void buildCentralGraphView();
    QCPGraph* instantiateNewGraph();
    void addCustomActionsToMenu() override;
    QCPGraph *getSelectedGraph();
    void setTheme();
    void onUpdateTimerTick();

private:
    Ui::GraphModule *ui;
    QCustomPlot graphCentralView;
    QMap<QString,QCPGraph*> subscriptions;

    int followedGraphIndex = -1;
    QString dateFormat = "%h:%m:%s (%z)"; //"HH:mm:ss\n(zzz)";
    //QSharedPointer<QCPAxisTickerDateTime> dateTicker;
    QTimer updaterTimer;
    int updatePeriod = 1000; // [ms]
    bool stopPlot = false;
};

#endif // GRAPHMODULE_H
