#include "customgraphicsscene.h"

#include <QGraphicsSceneMouseEvent>
#include <QGraphicsPixmapItem>
#include <QDebug>
#include <QDrag>
#include <QPainter>
#include <QMimeData>
#include <QApplication>
#include <QLabel>

CustomGraphicsScene::CustomGraphicsScene()
{

}

CustomGraphicsScene::~CustomGraphicsScene()
{
    for(int key : items.keys()){
        removeImageViewItem(key);
    }
}

void CustomGraphicsScene::addImageViewItem(const ImageViewerItem *item)
{
    if(items.contains(item->getId())){
        updateImageViewItem(item);
    }
    else{
        QGraphicsPixmapItem* itemView = createGraphicsView(item);
        addGraphicsItem(item->getId(), itemView);
    }
}

void CustomGraphicsScene::updateImageViewItem(const ImageViewerItem *item)
{
    if(items.contains(item->getId())){
        QGraphicsPixmapItem* itemView = items[item->getId()];
        if(itemView != nullptr){
            setupGraphicsItem(itemView, item);
        }
    }
}

void CustomGraphicsScene::removeImageViewItem(int id)
{
    if(items.contains(id)){
        QGraphicsItem *item = items[id];
        items.remove(id);
        delete item;
    }
}

void CustomGraphicsScene::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
{
    Q_UNUSED(event);
    isDragActive = false;
    emit dragStopped();
}

void CustomGraphicsScene::mousePressEvent(QGraphicsSceneMouseEvent *event)
{
    if(event->button() == Qt::LeftButton){
        dragStartPosition = event->scenePos();
        isDragActive = true;
        dragEmitted = false;
    }
}

void CustomGraphicsScene::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
{
    if(event->buttons() & Qt::LeftButton && isDragActive && !dragEmitted){
        emit dragStarted(dragStartPosition);
        dragEmitted = true;
    }

    emit mouseMoved(event->scenePos());

}

void CustomGraphicsScene::mouseDoubleClickEvent(QGraphicsSceneMouseEvent *event)
{
    emit doubleCLick(event->scenePos());
}

QGraphicsPixmapItem *CustomGraphicsScene::createGraphicsView(const ImageViewerItem *item) const
{
    QGraphicsPixmapItem *pixmapItem = new QGraphicsPixmapItem();
    setupGraphicsItem(pixmapItem, item);
    return pixmapItem;
}

void CustomGraphicsScene::setupGraphicsItem(QGraphicsPixmapItem *viewItem, const ImageViewerItem *item) const
{
    QPixmap pixmap;
    pixmap.loadFromData(item->toXmlObject().toXml().toUtf8());
    viewItem->setPixmap(pixmap);
    viewItem->setPos(item->getTopLeftCornerPosition());
}

void CustomGraphicsScene::addGraphicsItem(const int &id, QGraphicsPixmapItem *item)
{
    items[id] = item;
    addItem(item);
}
