#ifndef CUSTOMGRAPHICSSCENE_H
#define CUSTOMGRAPHICSSCENE_H

#include <QGraphicsScene>
#include <QGraphicsItem>
#include <QMap>

#include "imagevieweritem.h"

class CustomGraphicsScene : public QGraphicsScene
{
    Q_OBJECT

public:
    CustomGraphicsScene();
    ~CustomGraphicsScene();

    void addImageViewItem(const ImageViewerItem *item);
    void updateImageViewItem(const ImageViewerItem *item);
    void removeImageViewItem(int id);


signals:
    void leftMouseReleased(const QPointF &point);
    void mouseMoved(const QPointF &point);
    void dragStarted(const QPointF &initialPosition);
    void dragStopped();
    void doubleCLick(const QPointF &point);

protected:
    void mouseReleaseEvent(QGraphicsSceneMouseEvent *event) override;
    void mousePressEvent(QGraphicsSceneMouseEvent *event) override;
    void mouseMoveEvent(QGraphicsSceneMouseEvent* event) override;
    void mouseDoubleClickEvent(QGraphicsSceneMouseEvent* event) override;

    QGraphicsPixmapItem* createGraphicsView(const ImageViewerItem *item) const;
    void setupGraphicsItem(QGraphicsPixmapItem* viewItem, const ImageViewerItem *item) const;

    void addGraphicsItem(const int &id, QGraphicsPixmapItem *item);
//    void updateGraphicsItem(const ImageViewerItem &item, QGraphicsItem *viewItem);

private:
    QMap<int, QGraphicsPixmapItem*> items;
    QPointF dragStartPosition;
    bool isDragActive = false;
    bool dragEmitted = false;
};

#endif // CUSTOMGRAPHICSSCENE_H
