#ifndef IMAGEVIEWERITEM_H
#define IMAGEVIEWERITEM_H

#include <QString>
#include <QPoint>
#include <QSize>
#include "Core/xmlobject.h"
#include "Core/modulemessage.h"

class ImageViewerItem
{
public:
    ImageViewerItem();
    ImageViewerItem(const QPointF &center, const QSize &value);
    ImageViewerItem(const ImageViewerItem &other);

    void operator = (const ImageViewerItem &other );
    XmlObject toXmlObject() const;
    bool onMouseMoved(const QPointF &point);
    bool isPointInsideMyArea(const QPointF &point);
    void startDrag(const QPointF &initialPosition);
    void stopDrag();
    void onModuleMessageReceived(const ModuleMessage &msg);


    // getters/setters
    QString getTopic() const;
    QPointF getTopLeftCornerPosition() const;
    QSize getSize() const;
    void setSize(const QSize &value);
    void setSize(int width, int height);
    QString getText() const;
    void setText(const QString &value);
    void setCenter(const QPointF &center);
    int getId() const;
    void setId(int value);

//    bool getClicked() const;
//    void setClicked(bool value);

    QString getCurrentBorderColor() const;

    void setXml(const XmlObject &value);
    void setTopLeftCornerPosition(const QPointF &value);
    void setTopic(const QString &topic);

protected:
    void copy(const ImageViewerItem &other);
    void initialize();

    XmlObject *findOrAddXmlObject(const QString &xmlChildName);

private:
    int id = -1;
    QPointF topLeftCornerPosition;

//    const QString defaultBorderColor = "#000000";
//    const QString clickedBorderColor = "#32a83a";
    const QSize defaultSize = QSize(250, 150);
    const QString defaultText = "New item";

//    bool clicked = false;
    QPointF dragOffset;
    bool isDragActive = false;

    XmlObject xml;
};

#endif // IMAGEVIEWERITEM_H
