#ifndef IMAGEVIEWERMODULE_H
#define IMAGEVIEWERMODULE_H

#include "Core/module.h"
#include <QGraphicsScene>
#include <QGraphicsPixmapItem>
#include "Modules/DefaultModule/defaultmodule.h"
#include "Core/modulemessage.h"
#include "customgraphicsscene.h"
#include "imageviewersettings.h"

class ImageViewerModuleSettings;

namespace Ui {
class ImageViewerModule;
}

class ImageViewerModule : public DefaultModule
{
    Q_OBJECT

public:
    explicit ImageViewerModule(QWidget *parent = nullptr);
    ~ImageViewerModule();

    QWidget* toWidget() override;
    XmlObject toXmlObject() override;
    void fromXmlObject(const XmlObject &xmlObject) override;

    void showImage(const QString &filename);
    void showImage(const QPixmap &pixmap);
    void addItem(const XmlObject &xml);

public slots:

    void onItemTopicChanged(ImageViewerItem *item);
    void onReceiveMsg(const ModuleMessage &msg);
    void fitImage();
    void onImageFilePathChanged(const QString &newFilePath);
    void onItemDeleteRequested(int itemId);

protected:
    void resizeEvent(QResizeEvent *event) override;
    void addCustomActionsToMenu() override;
    void onChangeImageRequested();
    int onAddNewItemRequested();
    void onMouseMoved(const QPointF &point);
    void onDragStarted(const QPointF &initialPosition);
    void onDragStopped();
    void onDoubleClick(const QPointF &point);
    int addNewItem(const QPointF &point);
    int pickUnusedItemId();
    ImageViewerItem* createNewItem(const QPointF &point);
    int addItem(ImageViewerItem *item);
    void showInSettings(ImageViewerItem *item);
    void onSettingsSetupRequested(int id);
    void onSettingsSetupToAllRequested();

private:
    Ui::ImageViewerModule *ui;

    CustomGraphicsScene scene;
    QGraphicsPixmapItem backgroundImg;
    QString fileName = "overview.png";

    QMap<int, ImageViewerItem*> imageViewerItems;
    QMap<int, QString> topicMap; // Item id -> Topic
    int lastUsedId = 0;
    bool firstResizeDone = false;
    ImageViewerSettings settingsPanel;
    QPointF defaultCreationPoint = QPointF(200, 100);
};

#endif // IMAGEVIEWERMODULE_H
