#ifndef IMAGEVIEWERSETTINGS_H
#define IMAGEVIEWERSETTINGS_H

#include <QWidget>
#include "Core/xmlobject.h"
#include <QGraphicsScene>
#include <QGraphicsPixmapItem>
#include <QLineEdit>

namespace Ui {
class ImageViewerSettings;
}

class ImageViewerSettings : public QWidget
{
    Q_OBJECT

public:
    explicit ImageViewerSettings(QWidget *parent = nullptr);
    ~ImageViewerSettings();

    XmlObject getDefaultSettings() const;

    void setTarget(int value, XmlObject setup);

signals:
    void setupSettingsToAll();
    void setupSettingsToSelected(int id);
    void deleteItemRequested(int id);

protected:
    void initAndConnectUI();
    void defaultSettingsSetup();
    void updatePreview(bool updatePlainText = true);
    void onApplyToAllClicked();
    void onApplyToSelectedClicked();
    XmlObject* findOrAddXmlObject(const QString &xmlChild);
    void updateUI(XmlObject &xml);
    void resizeEvent(QResizeEvent *event) override;
    void fitPreview();
    void onDeleteClicked();

protected slots:
    void onPreviewTextChanged();

    void onPickColorRequested(QLineEdit *lineEdit);
    void onPropertyChanged();

private:
    Ui::ImageViewerSettings *ui;

    int target;
    XmlObject defaultSettings;
    QGraphicsScene scene;
    QGraphicsPixmapItem preview;
    bool lockUpdate = false;
};

#endif // IMAGEVIEWERSETTINGS_H
