#include "window.h"
#include "ui_window.h"

#include "Core/module.h"
#include "Core/modulesmanager.h"
#include <QVBoxLayout>

Window::Window(QWidget *parent) : QWidget(parent), ui(new Ui::Window)
{
    ui->setupUi(this);
}

Window::~Window()
{
    delete ui;
}

void Window::updateModule(Module *m)
{
    if(m != nullptr){
        if(centralModule == nullptr || centralModule != m){
            setCentralModule(m);
        }
    }
}

Module *Window::getCentralModule() const
{
    return centralModule;
}

void Window::setCentralModule(Module *value)
{
    if(centralModule != nullptr){
        //ui->mainLayout->removeWidget(centralModule->toWidget());
        disconnect(centralModule->getModuleEventsHandler(), &QWidget::destroyed, this, &Window::onWidgetDestroyed);
        disconnect(centralModule->getModuleEventsHandler(), &ModuleEventsHandler::replaceMeWith, this, &Window::onReplaceRequested);
    }
    centralModule = value;
    ui->mainLayout->addWidget(value->toWidget());
    connect(value->getModuleEventsHandler(), &QWidget::destroyed, this, &Window::onWidgetDestroyed);
    connect(value->getModuleEventsHandler(), &ModuleEventsHandler::replaceMeWith, this, &Window::onReplaceRequested);

}

void Window::closeEvent(QCloseEvent *event)
{
    if(centralModule != nullptr)
        delete centralModule;

    QWidget::closeEvent(event);

    deleteLater();
}

void Window::onWidgetDestroyed()
{
    centralModule = nullptr;
    close();
    this->deleteLater();
}

void Window::onReplaceRequested(Module *sender, Module *newModule)
{
    if(newModule != nullptr && sender != nullptr)
        setCentralModule(newModule);
}
