#include "mavlinkcommandadapter.h"

#include "Core/modulesmanager.h"

#include "Modules/skywardhubstrings.h"

MavlinkCommandAdapter::MavlinkCommandAdapter()
{

}

//void MavlinkCommandAdapter::setSerialPort(QSerialPort *value)
//{
//    serial = value;
//}

mavlink_message_t MavlinkCommandAdapter::encode_PING_TC()
{
    mavlink_message_t encoded_mvl_msg;
    auto unsignedStamp =  std::chrono::system_clock::now().time_since_epoch();
    std::chrono::milliseconds now = std::chrono::duration_cast<std::chrono::milliseconds>(unsignedStamp);
    std::uint64_t timeStamp = now.count();
    mavlink_msg_ping_tc_pack(MAV_SYS,MAV_CMP,&encoded_mvl_msg,timeStamp);
    return encoded_mvl_msg;
}

mavlink_message_t MavlinkCommandAdapter::encode_NOARG_TC(int cmd_id)
{
    mavlink_message_t encoded_mvl_msg;
    uint8_t command_id = cmd_id;
    mavlink_msg_noarg_tc_pack(MAV_SYS,MAV_CMP,&encoded_mvl_msg, command_id);
    return encoded_mvl_msg;
}

mavlink_message_t MavlinkCommandAdapter::encode_START_LAUNCH_TC(const ModuleMessage &msg)
{
    mavlink_message_t encoded_mvl_msg;
    int temp;
    if(msg.getIntPayload(temp)){
        uint64_t launch_code = temp;
        mavlink_msg_start_launch_tc_pack(MAV_SYS,MAV_CMP,&encoded_mvl_msg,launch_code);
    }
    return encoded_mvl_msg;
}

mavlink_message_t MavlinkCommandAdapter::encode_TELEMETRY_REQUEST_TC(uint8_t board_id)
{
    mavlink_message_t encoded_mvl_msg;
    mavlink_msg_telemetry_request_tc_pack(MAV_SYS,MAV_CMP,&encoded_mvl_msg, board_id);
    return encoded_mvl_msg;
}

mavlink_message_t MavlinkCommandAdapter::encode_RAW_EVENT_TC(const ModuleMessage &msg)
{
    mavlink_message_t encoded_mvl_msg;
    uint8_t event_id, topic_id;
    XmlObject msgOption = msg.getOptions();
    int temp;
    if(msgOption.getIntAttribute(SkywardHubStrings::raw_event_id,temp)){
        event_id = temp;
        if(msgOption.getIntAttribute(SkywardHubStrings::raw_event_topic_id,temp)){
            topic_id = temp;
            mavlink_msg_raw_event_tc_pack(MAV_SYS,MAV_CMP,&encoded_mvl_msg,event_id,topic_id);
        }
    }
    return encoded_mvl_msg;
}

mavlink_message_t MavlinkCommandAdapter::encode_AEROBRAKE_ANGLE_TC(const ModuleMessage &msg)
{
    mavlink_message_t encoded_mvl_msg;
    float angle;

    if(msg.getFloatPayload(angle)){
        mavlink_msg_set_aerobrake_angle_tc_pack(MAV_SYS,MAV_CMP,&encoded_mvl_msg, angle);
    }
    return encoded_mvl_msg;
}

mavlink_message_t MavlinkCommandAdapter::encode_REFERENCE_ALTITUDE_TC(const ModuleMessage &msg)
{
    mavlink_message_t encoded_mvl_msg;
    float ref_altitude;

    if(msg.getFloatPayload(ref_altitude)){
        mavlink_msg_set_reference_altitude_pack(MAV_SYS,MAV_CMP,&encoded_mvl_msg, ref_altitude);
    }
    return encoded_mvl_msg;
}

mavlink_message_t MavlinkCommandAdapter::encode_REFERENCE_TEMPERATURE_TC(const ModuleMessage &msg)
{
    mavlink_message_t encoded_mvl_msg;
    float ref_temp;

    if(msg.getFloatPayload(ref_temp)){
        mavlink_msg_set_reference_temperature_tc_pack(MAV_SYS,MAV_CMP,&encoded_mvl_msg, ref_temp);
    }
    return encoded_mvl_msg;
}

mavlink_message_t MavlinkCommandAdapter::encode_DEPLOYMENT_ALTITUDE_TC(const ModuleMessage &msg)
{
    mavlink_message_t encoded_mvl_msg;
    float ref_altitude;

    if(msg.getFloatPayload(ref_altitude)){
        mavlink_msg_set_deployment_altitude_tc_pack(MAV_SYS,MAV_CMP,&encoded_mvl_msg, ref_altitude);
    }
    return encoded_mvl_msg;
}

mavlink_message_t MavlinkCommandAdapter::encode_INITIAL_ORIENTATION_TC(const ModuleMessage &msg)
{
    mavlink_message_t encoded_mvl_msg;
    float yaw, pitch, roll;
    XmlObject msgOption = msg.getOptions();

    bool ok = msgOption.getFloatAttribute(SkywardHubStrings::mavlink_orientation_tc_yaw_name, yaw);
    ok = ok && msgOption.getFloatAttribute(SkywardHubStrings::mavlink_orientation_tc_pitch_name, pitch);
    ok = ok && msgOption.getFloatAttribute(SkywardHubStrings::mavlink_orientation_tc_roll_name, roll);

    if(ok){
        mavlink_msg_set_initial_orientation_tc_pack(MAV_SYS,MAV_CMP,&encoded_mvl_msg, yaw, pitch, roll);
    }
    return encoded_mvl_msg;
}

mavlink_message_t MavlinkCommandAdapter::encode_INITIAL_COORDINATES_TC(const ModuleMessage &msg)
{
    mavlink_message_t encoded_mvl_msg;
    float lat, longit;
    XmlObject msgOption = msg.getOptions();

    bool ok = msgOption.getFloatAttribute(SkywardHubStrings::mavlink_initial_coordinates_tc_latitude_name, lat);
    ok = ok && msgOption.getFloatAttribute(SkywardHubStrings::mavlink_initial_coordinates_tc_longitude_name, longit);

    if(ok){
        mavlink_msg_set_initial_coordinates_tc_pack(MAV_SYS,MAV_CMP,&encoded_mvl_msg, lat, longit);
    }
    return encoded_mvl_msg;
}

void MavlinkCommandAdapter::send(mavlink_message_t msg)
{
    ModuleMessage response(SkywardHubStrings::logCommandsTopic);
    response.addOption(SkywardHubStrings::msgIdField, QString::number(msg.msgid));
    response.addOption(SkywardHubStrings::msgSequenceNumberField, QString::number(msg.seq));

    if(portNumber >= 0){
        mavlinkWriter.setPortNumber(portNumber);
        mavlinkWriter.startAsyncWrite(msg);
    }
    else{
        response.setPayload(SkywardHubStrings::serialPortClosedErrorMsg);
    }

    emit publishRequested(response);
}

void MavlinkCommandAdapter::setPortNumber(int value)
{
    portNumber = value;
}


//void MavlinkCommandAdapter::loadMsgDefinition()
//{
//    XmlObject mavlinkMsgDef;
//    if(mavlinkMsgDef.loadFromFile(msgDefinitionFile)){
//        QList<XmlObject *> temp = mavlinkMsgDef.deepSearchObjects(mavlinkTm);
//        if(temp.count() == 1){
//            XmlObject *mavTmList = temp.first();
//            parseMavTmList(mavTmList);
//        }
//    }
//}

//void MavlinkCommandAdapter::parseMavTmList(const XmlObject *xml)
//{

//    for(int i = 0; i < xml->childCount(); i++){
//        XmlObject child = xml->childAt(i);
//        if(child.getObjectName() == childName
//                && child.hasAttribute(attributeName)
//                && child.hasAttribute(attributeValue)){

//        }
//    }
//}

bool MavlinkCommandAdapter::produceMsgFromXml(const XmlObject &xml, mavlink_message_t *msg)
{
    bool result = false;
    if(xml.getObjectName() == "ACK_TM"){
        QString recv_string = xml.getAttribute("recv_msgid");
        QString seq_string = xml.getAttribute("seq_ack");

        bool ok1, ok2;
        uint8_t recvId = recv_string.toUInt(&ok1);
        uint8_t seq = seq_string.toUInt(&ok2);

        if(ok1 && ok2){
            mavlink_msg_ack_tm_pack(MAV_SYS, MAV_CMP, msg, recvId, seq);
        }
        result = true;
    }

    return result;
}


























