#ifndef MAVLINKMODULE_H
#define MAVLINKMODULE_H

#include <QWidget>
#include "Core/module.h"
//#include "Modules/Graph/qcustomplot.h"
#include "Modules/Utility/togglebutton.h"
#include <QTimer>
#include "Core/modulemessage.h"
#include "mavlinkreader.h"
#include "mavlinkcommandadapter.h"
#include <QSerialPort>
#include "Modules/DefaultModule/defaultmodule.h"

Q_DECLARE_METATYPE(ModuleMessage);

namespace Ui {
class MavlinkModule;
}

class MavlinkModule : public DefaultModule
{
    Q_OBJECT

public:
    explicit MavlinkModule(QWidget *parent = nullptr);
    ~MavlinkModule();

    QWidget * toWidget() override;
    XmlObject toXmlObject() override;
    void fromXmlObject(const XmlObject &xmlObject) override;
//    void replotAll();

    void onCommandReceived(const ModuleMessage &msg);
    void onTelemetryRequestReceived(const ModuleMessage &msg);

public slots:
    void onStartClicked();
    void onStopClicked();
    void onMsgReceived(const ModuleMessage &msg);
    void onLinkQualityIndicatorClicked(bool checked);
    void onLinkQualityTimerTick();

protected:
    void connectUiSlots();
    void initializeSerialPortView();
//    void buildCentralGraphView();
    mavlink_message_t parseMavlinkMsg(char *buffer, int readCount);

    void subscribe();
    void publish(const ModuleMessage &msg);

    void closePort();
    void updateLinkSignalIndicator();

    void updateLinkSignalIndicator(int msgArrived, float ratio);
    void onOpenLogFolderClick();

private slots:
    void onSerialPortIndexChanged(int index);
    bool startReadingOnSerialPort();
    void onStartStreamToggled(bool state);


private:
    Ui::MavlinkModule *ui;

//    QCustomPlot graphCentralView;
//    QCPGraph *linkGraph = nullptr;
    ToggleButton startSerialStreamToggleButton;
    QString dateFormat = "HH:mm:ss\n(zzz)";
    MavlinkReader mavlinkReader;
    MavlinkCommandAdapter mavlinkCommandAdapter;
    //QSerialPort serialPort;

    QTimer linkQualityTimer;
    int linkQualityPeriod = 1000; // [ms]
    int msgArrived = 0;
    bool portOpen = false;
    int portNumber = -1;
};

#endif // MAVLINKMODULE_H
