#include "mavlinkreader.h"

#include <QFile>
#include <QTextStream>
#include <QDebug>
#include "windows.h"
#include "rs232.h"
#include "Modules/skywardhubstrings.h"
#include <QDir>
#include "mavlinkcommandadapter.h"

MavlinkReader::MavlinkReader()
{
    //loadMsgMapping();
}

MavlinkReader::~MavlinkReader()
{
    closeLog();
}

void MavlinkReader::startAsyncReading()
{
    stop = false;
    this->start(); // Chiama run()
}

void MavlinkReader::startAsyncReadingWithSignal(QSerialPort *port)
{
    stop = false;
    serial = port;

    connect(serial, &QSerialPort::readyRead, this, &MavlinkReader::handleReadyRead);
    connect(serial, &QSerialPort::errorOccurred, this, &MavlinkReader::handleError);
}

//void MavlinkReader::loadMsgMapping()
//{
//    QFile msgDefinitionFile(mavlinkMsgDefinitionFilePath);
//    if(msgDefinitionFile.open(QIODevice::ReadOnly)){
//        msgDefinition.fromXml(QTextStream(&msgDefinitionFile).readAll());

//        //Per provare e vedere se la riserializzazione funziona
////        QFile file("GroundStationConfig/prova.xml");

////        if(file.open(QIODevice::WriteOnly)){
////            QTextStream(&file) << msgDefinition.toXml();
////        }
//    }
//}

void MavlinkReader::run()
{
    bool ok;
    while (!stop) {
        mavlink_message_t msg = waitForMavlinkMsg(ok);

        if(ok && chackMsgOwnership(msg)){
            parseMavlinkMsg(&msg);
        }
    }
}

bool MavlinkReader::chackMsgOwnership(const mavlink_message_t& msg) const
{
    return true;
//    if(msg.sysid == MavlinkCommandAdapter::MAV_SYS  && msg.compid == MavlinkCommandAdapter::MAV_CMP){
//        return true;
//    }
//    return false;
}


void MavlinkReader::setPortNumber(int value)
{
    portNumber = value;
}

void MavlinkReader::stopReading()
{
    stop = true;
    closeLog();
}

mavlink_message_t MavlinkReader::waitForMavlinkMsg(bool &ok)
{
    mavlink_message_t msg;

    //ok = readWithQSerialPort(&msg);
    ok = readWithRS232(&msg);

    return msg;
}

bool MavlinkReader::readWithRS232(mavlink_message_t *msg)
{
    mavlink_status_t status;
    unsigned char byte;

    int numRead = 0;
    while (!stop) {
        numRead = RS232_PollComport(portNumber,&byte,1);
        //qDebug() << byte;

        if(numRead > 0){

            if(logFile.isOpen())
            {
                const char c = byte;
                logFile.write(&c, sizeof(char));
            }

            // add byte to parser
            if(mavlink_parse_char(MAVLINK_COMM_0, byte, msg, &status)){
                // if the parser has found a message, return it
                return true;
            }
        }
    }

    return false;
}

bool MavlinkReader::readWithQSerialPort(mavlink_message_t *msg)
{
    mavlink_status_t status;
    int waitingTimeout = 1000;
    unsigned char byte;

    int numRead = 0;

    while (!stop && serial != nullptr && serial->waitForReadyRead(waitingTimeout)) {

        QByteArray dataRead = serial->readAll();
        numRead = dataRead.size();
        for(int i = 0; i < dataRead.count(); i++){
            byte = dataRead.at(i);
//            qDebug() << "MavReader: " << byte;

            if(mavlink_parse_char(MAVLINK_COMM_0, byte, msg, &status)){
                // if the parser has found a message, return it
                return true;
            }
        }

        if(numRead == -1){
            //An error occurred
            stop = true;
        }
    }

    return false;
}


void MavlinkReader::handleReadyRead()
{

    if(stop)
        return;

    mavlink_message_t msg;
    mavlink_status_t status;


    QByteArray dataRead = serial->readAll();
    int numRead = dataRead.count();
    for(int i = 0; i < numRead; i++){
        unsigned char byte = dataRead.at(i);

        if(mavlink_parse_char(MAVLINK_COMM_0, byte, &msg, &status)){
            // if the parser has found a message, return it
            parseMavlinkMsg(&msg);
        }
    }
}

void MavlinkReader::handleError()
{
    qDebug() << "MavlinkReader >> Serial read error";
}


QList<ModuleMessage> MavlinkReader::parseMavlinkMsg(mavlink_message_t *msg)
{
    if(msg == nullptr)
        return QList<ModuleMessage>();

    const mavlink_message_info_t *m = &message_info[msg->msgid];
    const mavlink_field_info_t *f = m->fields;
    unsigned i;

    QMap<QString,QString> fields;

    for (i = 0; i < m->num_fields; i++) {
        // For each field of the message
        QString payload = extractFields(msg, &f[i]);
        fields[QString(f[i].name)] = payload;
    }
    QString msgName = QString(m->name);
    return translateToGsMessage(msg, msgName, fields);
}

QString MavlinkReader::extractFields(mavlink_message_t *msg, const mavlink_field_info_t *f)
{
    QString payload;

    if (f->array_length == 0) {
        payload.append(extractOneField(msg, f, 0));
    }
    else{
        unsigned i;
        /* print an array */
        if (f->type == MAVLINK_TYPE_CHAR) {
            //printf("'%.*s'", f->array_length,f->wire_offset+(const char *)_MAV_PAYLOAD(msg));
            payload.append(QString((const char *)_MAV_PAYLOAD(msg)));
        }
        else{
            for (i=0; i<f->array_length; i++) {
                 payload.append(extractOneField(msg, f, i));
            }
        }
    }

    return payload;
}

QList<ModuleMessage> MavlinkReader::translateToGsMessage(mavlink_message_t *msg, const QString &msgName, const QMap<QString,QString> &payloads)
{
    QTime timestamp(0, 0, 0, 0);
    bool timeOK = false;
    double t;
    if(payloads.contains("timestamp")){
        t = payloads["timestamp"].toDouble(&timeOK);
    }

    if(timeOK){
        timestamp = timestamp.addMSecs(t);
    }

    QList<ModuleMessage> messagesReceived;

    for(QString fieldName : payloads.keys()){
        QString topic = SkywardHubStrings::mavlink_received_msg_topic + "/" + msgName + "/" + fieldName;
        ModuleMessage msg(topic, payloads[fieldName], timestamp);
        emit msgReceived(msg);
        messagesReceived.append(msg);
    }
    return messagesReceived;
}


//QDateTime MavlinkReader::convertTimestampToDatatime(const double &timestamp) const
//{
//    // The timestamp measure the ms from the start of the board.

////    double secondsFromStart = timestamp/1000;
////    double minutes = secondsFromStart/60;
////    double hours = minutes/60;

//    QDateTime date;
//    return date.addMSecs(timestamp);
//}


void MavlinkReader::closeLog()
{
    if(logFile.isOpen())
        logFile.close();
}

void MavlinkReader::openLogFile()
{
    closeLog();

    setLogFilePath();

    logFile.setFileName(logFilePath);
    if (logFile.open(QIODevice::WriteOnly)){
        qDebug() << "MavlinkReader >> Opened " << logFilePath;
        QTextStream out(&logFile);
        out << "Log started " << QDateTime::currentDateTime().toString() << "\n";
    }
    else{
        qDebug() << "MavlinkReader >> Error, can not open " << logFilePath;
    }
}

void MavlinkReader::setLogFilePath()
{
    QDir dir(SkywardHubStrings::defaultLogsFolder);
    if (!dir.exists())
        dir.mkpath(".");

    QString extension = ".dat";
    QString currentFile =  QDateTime::currentDateTime().toString("yyyyMMdd_hh.mm.ss");
    QString proposedFilePath = SkywardHubStrings::defaultLogsFolder + "/" + currentFile;
    QString fileNumber = "";
    int i = 1;
    while(QFile::exists(proposedFilePath+fileNumber+extension) && i < 200){
        i++;
        fileNumber = " ("+QString::number(i)+")";
    }

    logFilePath = proposedFilePath+fileNumber+extension;
}


QString MavlinkReader::extractOneField(mavlink_message_t *msg, const mavlink_field_info_t *f, int idx)
{
    //#define PRINT_FORMAT(f, def) (f->print_format?f->print_format:def)
    QString payload = "";
    switch (f->type) {
        case MAVLINK_TYPE_CHAR:
        {
         //printf(PRINT_FORMAT(f, "%c"), _MAV_RETURN_char(msg, f->wire_offset+idx*1));
         char val_char = _MAV_RETURN_char(msg, f->wire_offset+idx*1);
         payload = QString(val_char);
        } break;
        case MAVLINK_TYPE_UINT8_T:
        {
         //printf(PRINT_FORMAT(f, "%u"), _MAV_RETURN_uint8_t(msg, f->wire_offset+idx*1));
         uint8_t val_uint8 = _MAV_RETURN_uint8_t(msg, f->wire_offset+idx*1);
         payload = QString::number(val_uint8);

        } break;
        case MAVLINK_TYPE_INT8_T:
        {
         //printf(PRINT_FORMAT(f, "%d"), _MAV_RETURN_int8_t(msg, f->wire_offset+idx*1));
         int8_t val_int8 = _MAV_RETURN_int8_t(msg, f->wire_offset+idx*1);
         payload = QString::number(val_int8);

        } break;
        case MAVLINK_TYPE_UINT16_T:
        {
         //printf(PRINT_FORMAT(f, "%u"), _MAV_RETURN_uint16_t(msg, f->wire_offset+idx*2));
         uint16_t val_uint16 = _MAV_RETURN_uint16_t(msg, f->wire_offset+idx*2);
         payload = QString::number(val_uint16);

        } break;
        case MAVLINK_TYPE_INT16_T:
        {
        //printf(PRINT_FORMAT(f, "%d"), _MAV_RETURN_int16_t(msg, f->wire_offset+idx*2));
        int16_t val_int16 = _MAV_RETURN_int16_t(msg, f->wire_offset+idx*2);
        payload = QString::number(val_int16);

        } break;
        case MAVLINK_TYPE_UINT32_T:
        {
        //printf(PRINT_FORMAT(f, "%lu"), (unsigned long)_MAV_RETURN_uint32_t(msg, f->wire_offset+idx*4));
        unsigned long val_uint32 = (unsigned long)_MAV_RETURN_uint32_t(msg, f->wire_offset+idx*4);
        payload = QString::number(val_uint32);

        } break;
        case MAVLINK_TYPE_INT32_T:
        {
        //printf(PRINT_FORMAT(f, "%ld"), (long)_MAV_RETURN_int32_t(msg, f->wire_offset+idx*4));
        long val_int32 = (long)_MAV_RETURN_int32_t(msg, f->wire_offset+idx*4);
        payload = QString::number(val_int32);

        } break;
        case MAVLINK_TYPE_UINT64_T:
        {
        //printf(PRINT_FORMAT(f, "%llu"), (unsigned long long)_MAV_RETURN_uint64_t(msg, f->wire_offset+idx*8));
        unsigned long long val_uint64 = (unsigned long long)_MAV_RETURN_uint64_t(msg, f->wire_offset+idx*8);
        payload = QString::number(val_uint64);

        } break;
        case MAVLINK_TYPE_INT64_T:
        {
        //printf(PRINT_FORMAT(f, "%lld"), (long long)_MAV_RETURN_int64_t(msg, f->wire_offset+idx*8));
        long long val_int64 = (long long)_MAV_RETURN_int64_t(msg, f->wire_offset+idx*8);
        payload = QString::number(val_int64);

        } break;
        case MAVLINK_TYPE_FLOAT:
        {
        //printf(PRINT_FORMAT(f, "%f"), (double)_MAV_RETURN_float(msg, f->wire_offset+idx*4));
        double val_double = (double)_MAV_RETURN_float(msg, f->wire_offset+idx*4);
        QTextStream out(&payload);
        out.setRealNumberNotation(QTextStream::FixedNotation);
        out << val_double;

        } break;
        case MAVLINK_TYPE_DOUBLE:
        {
        //printf(PRINT_FORMAT(f, "%f"), _MAV_RETURN_double(msg, f->wire_offset+idx*8));
        double val_double2 = _MAV_RETURN_double(msg, f->wire_offset+idx*8);
        payload = QString::number(val_double2);

        } break;
    }

    return payload;
}


