#ifndef MAVLINKREADER_H
#define MAVLINKREADER_H

#include <QSerialPort>
#include <QThread>
#include <QDateTime>
#include "Core/xmlobject.h"
#include "mavlinkversionheader.h"

#include "Core/modulemessage.h"
#include <QFile>


class MavlinkReader : public QThread
{
    Q_OBJECT

public:
    MavlinkReader();
    ~MavlinkReader();

    void startAsyncReading();
    void startAsyncReadingWithSignal(QSerialPort *port);

    QList<ModuleMessage> parseMavlinkMsg(mavlink_message_t *msg);
    void stopReading();

    void setPortNumber(int value);
    void closeLog();
    void openLogFile();

signals:
    //void receivedMsg(const QString &topic,const QString &payload,const QDateTime &timestamp);
    void msgReceived(const ModuleMessage &msg);

protected:
    //void loadMsgMapping();
    mavlink_message_t waitForMavlinkMsg(bool &ok);

    QString extractFields(mavlink_message_t *msg, const mavlink_field_info_t *f);
    QString extractOneField(mavlink_message_t *msg, const mavlink_field_info_t *f, int idx);

    bool readWithQSerialPort(mavlink_message_t *msg);
    bool readWithRS232(mavlink_message_t *msg);

    void handleReadyRead();
    void handleError();
    bool chackMsgOwnership(const mavlink_message_t& msg) const;

    void setLogFilePath();
//    QDateTime convertTimestampToDatatime(const double &timestamp) const;

    /*
     * This method emit the signal msgReceived
     */
    QList<ModuleMessage> translateToGsMessage(mavlink_message_t *msg,  const QString &msgName, const QMap<QString, QString> &payloads);
    void run();

private:
    QSerialPort *serial = nullptr;
    bool stop = false;
    const mavlink_message_info_t message_info[256] = MAVLINK_MESSAGE_INFO;
    int portNumber = -1;
    //XmlObject msgDefinition;
    //QString mavlinkMsgDefinitionFilePath = "GroundStationConfig/mavlinkMsgDefinition.xml";

    QString logFilePath = "";
    QFile logFile;
};

#endif // MAVLINKREADER_H
