#include "messagesviewermodule.h"
#include "ui_messagesviewermodule.h"

#include "Core/modulemessagesbroker.h"
#include "Modules/skywardhubstrings.h"
#include <QTableWidgetItem>

MessagesViewerModule::MessagesViewerModule(QWidget *parent) : DefaultModule(parent), ui(new Ui::MessagesViewerModule)
{
    ui->setupUi(this);
    defaultContextMenuSetup();

    getCore()->getModuleMessagesBroker()->subscribe(SkywardHubStrings::commandsTopic, this, [this](const ModuleMessage &msg){
        addMsgSent(msg);
    });

    getCore()->getModuleMessagesBroker()->subscribe(ackTopic, this, [this](const ModuleMessage &msg){
        handleAck(msg);
    });
}

MessagesViewerModule::~MessagesViewerModule()
{
    delete ui;
}

QWidget *MessagesViewerModule::toWidget()
{
    return this;
}

XmlObject MessagesViewerModule::toXmlObject()
{
    return XmlObject(getName(ModuleId::MESSAGEVIEWER));
}

void MessagesViewerModule::fromXmlObject(const XmlObject &xmlObject)
{
    Q_UNUSED(xmlObject);
}

void MessagesViewerModule::addMsgSent(const ModuleMessage &msg)
{
    int row = updateVerticalHeaders(msg);
    int column = 0;

    QString msgName = computeMsgName(msg);
    QTableWidgetItem *newItem = new QTableWidgetItem(msgName);
    newItem->setBackground(QBrush(QColor(255, 153, 51)));
    newItem->setForeground(QBrush(QColor(0,0,0)));
    ui->tableWidget->setItem(row,column,newItem);
    registerMessage(newItem,msg);
}

QString MessagesViewerModule::computeMsgName(const ModuleMessage &msg)
{
    return msg.originalTopic().replace(SkywardHubStrings::commandsTopic+"/", "");
}

void MessagesViewerModule::handleAck(const ModuleMessage &ack)
{
    for(int i=0; i < messages.count(); i++){
        ModuleMessage msg = messages[i].getMsg();

        // If the two messages have the same ack and sequence number
        if(msg.getOptions().getAttribute(SkywardHubStrings::msgIdField) ==         // THE NAMES OF THE FIELDS SHOULD BE LOADED FROM THE MESSAGES DEFINITION XML
                ack.getOptions().getAttribute(SkywardHubStrings::msgIdField)){
            if(msg.getOptions().getAttribute(SkywardHubStrings::msgSequenceNumberField) ==
                    ack.getOptions().getAttribute(SkywardHubStrings::msgSequenceNumberField)){
                messages[i].getItem()->setBackground(QBrush(QColor(0, 255, 255)));
            }
        }
    }
}

int MessagesViewerModule::updateVerticalHeaders(const ModuleMessage &msg)
{
    int row = 0; // Insert on top
    QString headerTxt = msg.timestamp().toString("hh.mm.ss (zzz)");
    ui->tableWidget->insertRow(row);
    QTableWidgetItem *header = new QTableWidgetItem(headerTxt);
    ui->tableWidget->setVerticalHeaderItem(row,header);

    int rowCount = ui->tableWidget->rowCount();
    if( rowCount > maxSize && rowCount > 0){
        ui->tableWidget->removeRow(rowCount - 1);
    }

    return row;
}

void MessagesViewerModule::registerMessage(QTableWidgetItem *item, const ModuleMessage &msg)
{
    messages.prepend(MessageLog(item, msg));
    if(messages.count() > maxSize){
        messages.removeLast();
    }
}



// ____________________ MESSAGE LOG CLASS _______________________________

MessageLog::MessageLog(QTableWidgetItem *item, const ModuleMessage &msg)
{
    setItem(item);
    setMsg(msg);
}

QTableWidgetItem *MessageLog::getItem() const
{
    return item;
}

void MessageLog::setItem(QTableWidgetItem *value)
{
    item = value;
}

ModuleMessage MessageLog::getMsg() const
{
    return msg;
}

void MessageLog::setMsg(const ModuleMessage &value)
{
    msg = value;
}
