#ifndef MESSAGESVIEWERMODULE_H
#define MESSAGESVIEWERMODULE_H

#include <QWidget>
#include "Core/module.h"
#include "Core/modulemessage.h"
#include "Modules/DefaultModule/defaultmodule.h"

class QTableWidgetItem;

class MessageLog{

public:
    explicit MessageLog(QTableWidgetItem *item, const ModuleMessage &msg);

    QTableWidgetItem *getItem() const;
    void setItem(QTableWidgetItem *value);

    ModuleMessage getMsg() const;
    void setMsg(const ModuleMessage &value);

private:
    QTableWidgetItem *item = nullptr;
    ModuleMessage msg;
};


namespace Ui {
class MessagesViewerModule;
}

class MessagesViewerModule : public DefaultModule
{
    Q_OBJECT

public:
    explicit MessagesViewerModule(QWidget *parent = nullptr);
    ~MessagesViewerModule();

    QWidget *toWidget() override;

    XmlObject toXmlObject() override;
    void fromXmlObject(const XmlObject &xmlObject) override;

    void addMsgSent(const ModuleMessage &msg);
    void handleAck(const ModuleMessage &ack);

protected:
    int updateVerticalHeaders(const ModuleMessage &msg);
    void registerMessage(QTableWidgetItem *item, const ModuleMessage &msg);
    QString computeMsgName(const ModuleMessage &msg);

private:
    Ui::MessagesViewerModule *ui;
    QList<MessageLog> messages;
    QString ackTopic = SkywardHubStrings::mavlink_received_msg_ACK_topic;
    int maxSize = 10;
};

#endif // MESSAGESVIEWERMODULE_H
