#include "scrollareamodule.h"
#include "ui_scrollareamodule.h"

#include "Core/module.h"
#include "Core/modulesmanager.h"
#include "Core/moduleeventshandler.h"

ScrollAreaModule::ScrollAreaModule(QWidget *parent) : DefaultModule(parent), ui(new Ui::ScrollAreaModule)
{
    ui->setupUi(this);
    defaultContextMenuSetup();
}

ScrollAreaModule::~ScrollAreaModule()
{
    clearModuleList();
    delete ui;
}


QWidget *ScrollAreaModule::toWidget()
{
    return this;
}

XmlObject ScrollAreaModule::toXmlObject()
{
    return XmlObject(getName(ModuleId::SCROLLAREA));
}

void ScrollAreaModule::fromXmlObject(const XmlObject &xmlObject)
{
    Q_UNUSED(xmlObject);
}

void ScrollAreaModule::clearModuleList()
{
    QList<Module*> list = this->modulesList;
    for(int i = 0; i <list.count(); i++){
        delete list[i];
    }
    modulesList.clear();
}

void ScrollAreaModule::onModuleBeforeDelete(Module *module)
{
    if(module != nullptr){
        modulesList.removeAll(module);
    }
}

void ScrollAreaModule::addCustomActionsToMenu()
{
    QAction* addItem = new QAction("Add Item");
    connect(addItem, &QAction::triggered,this, &ScrollAreaModule::onAddItemClicked);
    addActionToMenu(addItem);
}

void ScrollAreaModule::onAddItemClicked()
{
    Module* module = getCore()->getModulesManager()->instantiateDefaultModule();
    addModule(module);
}


void ScrollAreaModule::addModule(Module *module, int position)
{
    ui->scrollArea_mainLayout->insertWidget(position, module->toWidget());
    if(position < modulesList.count()){
        modulesList.insert(position, module);
    }
    else{
        modulesList.append(module);
    }
    connect(module->getModuleEventsHandler(),&ModuleEventsHandler::beforeDelete, this, &ScrollAreaModule::onModuleBeforeDelete);
    connect(module->getModuleEventsHandler(),&ModuleEventsHandler::contextMenuRequest,this,&ScrollAreaModule::onSkywardHubContextMenuRequested);
    connect(module->getModuleEventsHandler(),&ModuleEventsHandler::replaceMeWith, [this](Module* sender, Module* newModule){
        this->replace(sender,newModule);
    });
}

void ScrollAreaModule::replace(Module *oldModule, Module *newModule)
{
    int indexOfModule = modulesList.indexOf(oldModule);
    if(indexOfModule < 0)
        return;

    modulesList[indexOfModule] = newModule;
    addModule(newModule, indexOfModule);
}
