#include "prefabviewelement.h"
#include "ui_prefabviewelement.h"

#include "Modules/skywardhubstrings.h"

PrefabViewElement::PrefabViewElement(const QString &name, QWidget* parent) : QWidget(parent), ui(new Ui::PrefabViewElement)
{
    ui->setupUi(this);
    setName(name);

    connect(ui->pushButton_launch, &QPushButton::clicked, this, &PrefabViewElement::onLaunchClicked);
    connect(ui->pushButton_delete, &QPushButton::clicked, this, &PrefabViewElement::onDeleteClicked);
    connect(ui->pushButton_icon, &QPushButton::clicked, this, &PrefabViewElement::onIconClicked);
}

PrefabViewElement::~PrefabViewElement()
{
    delete ui;
}

void PrefabViewElement::setName(const QString &value)
{
    name = value;
    ui->label_name->setText(value);
}

QString PrefabViewElement::getName() const
{
    return name;
}

QString PrefabViewElement::getDesctiption() const
{
    return desctiption;
}

void PrefabViewElement::setDesctiption(const QString &value)
{
    if(value.isEmpty())
        return;

    desctiption = value;
    ui->label_desctiption->setText(value);
}

QString PrefabViewElement::getPathToImg() const
{
    return pathToImg;
}

void PrefabViewElement::setPathToImg(const QString &value)
{
    if(value.isEmpty())
        return;

    pathToImg = value;
    QPixmap pixmap(value);
    QIcon ButtonIcon(pixmap);
    ui->pushButton_icon->setIcon(ButtonIcon);
    ui->pushButton_icon->setIconSize(pixmap.rect().size());
    ui->pushButton_icon->setIconSize(QSize(512, 512));
}

XmlObject PrefabViewElement::getXml() const
{
    return xml;
}

void PrefabViewElement::setXml(XmlObject &xml)
{
    this->xml = xml;

    setPathToImg(xml.getAttribute(SkywardHubStrings::configurationIconPathAttribute));

    QList<XmlObject*> descr = xml.deepSearchObjects(SkywardHubStrings::configurationDescriptionAttribute);

    if(descr.count() >= 1){
        setDesctiption(descr.first()->getTextValue());
    }

}

void PrefabViewElement::addActiveModuleName(const QString &moduleName)
{
    if(containsModule(moduleName))
        return;

    activeModules.append(moduleName);
}

void PrefabViewElement::removeActiveModuleName(const QString &moduleName)
{
    activeModules.removeAll(moduleName);
}

bool PrefabViewElement::containsModule(const QString &moduleName)
{
    return activeModules.contains(moduleName);
}

void PrefabViewElement::setDeleteButtonVisibility(bool visibility)
{
    ui->pushButton_delete->setVisible(visibility);
}

void PrefabViewElement::setLaunchButtonText(const QString &txt)
{
    ui->pushButton_launch->setText(txt);
}

void PrefabViewElement::setIconButtonVisibility(bool visibility)
{
    ui->pushButton_icon->setVisible(visibility);
}

void PrefabViewElement::onIconClicked()
{
    emit selected(this);
}

void PrefabViewElement::onLaunchClicked()
{
    emit launchRequested(this);
}

void PrefabViewElement::onDeleteClicked()
{
    emit deleteRequested(this);
}

QString PrefabViewElement::getFilePath() const
{
    return filePath;
}

void PrefabViewElement::setFilePath(const QString &value)
{
    filePath = value;
}

