#ifndef TABLEMODULE_H
#define TABLEMODULE_H

#include <QWidget>
#include <QStringList>
#include "Core/module.h"
#include "Core/modulemessage.h"
#include "Modules/DefaultModule/defaultmodule.h"


namespace Ui {
class TableModule;
}

class TableModule : public DefaultModule
{
    Q_OBJECT

public:
    explicit TableModule(QWidget *parent = nullptr);
    ~TableModule();

    QWidget *toWidget() override;    
    XmlObject toXmlObject() override;
    void fromXmlObject(const XmlObject &xmlObject) override;

public slots:
    void onSubscribeClicked();
    void onSubscriptionAdded(const QString &subscription);
    void onSubscriptionRemoved(const QString &subscription);
    void onMsgReceived(const ModuleMessage &msg);
    void onColumnNumberChanged(int columnNumber);
    void onTableTitleRenameRequest();

protected:
    void connectUiSlots();
    int updateVerticalHeaders(const QString &txt);
    int updateHorizontalHeaders(const QString &txt);
    void removeRow(const QString &txt);
    void addCustomActionsToMenu() override;

private:
    Ui::TableModule *ui;
    QStringList subscriptions;
    QStringList verticalHeaders;
    QStringList horizontalHeaders;
    QString horizontalHeaderDateFormat = "HH:mm:ss\n(zzz)";
};

#endif // TABLEMODULE_H
