#include "testmodule.h"
#include "ui_testmodule.h"

#include "Modules/Utility/contextmenuseparator.h"
#include "Core/modulesmanager.h"
#include "Core/modulemessagesbroker.h"

TestModule::TestModule(QWidget *parent) : DefaultModule(parent), ui(new Ui::TestModule)
{
    ui->setupUi(this);
    defaultContextMenuSetup();

    maxChar = ui->maxChar_spinBox->value();
    connect(ui->maxChar_spinBox,SIGNAL(valueChanged(int)),this,SLOT(onMaxCharChanged(int)));
}

TestModule::~TestModule()
{
    delete ui;
}

QWidget* TestModule::toWidget()
{
    return this;
}

XmlObject TestModule::toXmlObject()
{
    return XmlObject(getName(ModuleId::BROKERTEST));
}

void TestModule::fromXmlObject(const XmlObject &xmlObject)
{
    Q_UNUSED(xmlObject);
}

void TestModule::on_publish_button_clicked()
{
    QString topic = ui->topic_lineEdit->text().trimmed();
    QString payload = ui->payload_lineEdit->text().trimmed();
    ModuleMessage msg(topic,payload, QTime::currentTime());
    getCore()->getModuleMessagesBroker()->publish(msg);
}

void TestModule::on_subscribe_button_clicked()
{
    getCore()->getModuleMessagesBroker()->subscribe(ui->subscribe_lineEdit->text().trimmed(),this,[this](const ModuleMessage &msg){
        QString oldText = ui->textArea->toPlainText();
        QString line = QDateTime::currentDateTime().toString("hh.mm.ss (zzz): ") + "(Topic) " + msg.originalTopic() + " -> " + msg.payload() + " (payload)";
        auto i = msg.getOptions().attributesIterator();
        while(i.hasNext() && line.count() < maxChar){
            i.next();
            line = line + ", " + i.value() + " ("+i.key()+")";
        }
        QString newText(line + '\n' + oldText);
        ui->textArea->setPlainText(newText.mid(0, maxChar));
    });
}

void TestModule::on_unsubscribe_button_clicked()
{
    getCore()->getModuleMessagesBroker()->unsubscribe(ui->subscribe_lineEdit->text().trimmed(),this);
}

void TestModule::onMaxCharChanged(int value)
{
    maxChar = value;
}
