#include "timercontrollermodule.h"
#include "ui_timercontrollermodule.h"

#include <QFile>
#include <QTextStream>

TimerControllerModule::TimerControllerModule(QWidget *parent) : DefaultModule(parent), ui(new Ui::TimerControllerModule)
{
    ui->setupUi(this);

    connect(ui->setTimer_pushButton, &QPushButton::clicked, this, &TimerControllerModule::onSetTimerClicked);
    connect(ui->setLabelInfo_pushButton, &QPushButton::clicked, this, &TimerControllerModule::onSetInfoLabelClicked);
    connect(ui->start_pushButton, &QPushButton::clicked, this, &TimerControllerModule::onStartTimerClicked);
    connect(ui->hold_pushButton, &QPushButton::clicked, this, &TimerControllerModule::onHoldTimerClicked);
    connect(ui->enableOutput_checkBox, &QCheckBox::toggled, this, &TimerControllerModule::onEnableOutputToggled);
    connect(&timer, &QTimer::timeout, this, &TimerControllerModule::onTimerTick);

    initCurrentTime();
}

TimerControllerModule::~TimerControllerModule()
{
    delete ui;
}

QWidget *TimerControllerModule::toWidget()
{
    return this;
}

XmlObject TimerControllerModule::toXmlObject()
{
    return XmlObject(getName(ModuleId::TIMER_CONTROLLER));
}

void TimerControllerModule::fromXmlObject(const XmlObject &xmlObject)
{
    Q_UNUSED(xmlObject);
}

void TimerControllerModule::onSetTimerClicked()
{
    initCurrentTime();
}

void TimerControllerModule::onStartTimerClicked()
{
    isHold = false;
    if(!timer.isActive()){
        timer.start(timerIntervalMs);
    }
    setInfoLabel("");
}

void TimerControllerModule::initCurrentTime()
{
    int hours = ui->hours_spinBox->value();
    int minutes = ui->minutes_spinBox->value();
    int seconds = ui->seconds_spinBox->value();

    currentTime = QTime(hours, minutes, seconds);

    if(ui->increment_comboBox->currentIndex() == 0){
        increment = -1;
    }
    else{
        increment = 1;
    }
    updateTimeLabel();
}

void TimerControllerModule::onHoldTimerClicked()
{
    timer.stop();
    isHold = true;
    setInfoLabel(holdTxt);
}

void TimerControllerModule::onSetInfoLabelClicked()
{
    setInfoLabel(ui->labelInfo_lineEdit->text());
}

void TimerControllerModule::onTimerTick()
{
    currentTime = currentTime.addSecs(getIncrement());
    QString output = updateTimeLabel();
    updateOutputFile(output);

    if(isTimeExpired()){
        increment = increment * -1;
    }
}

void TimerControllerModule::onEnableOutputToggled(bool val)
{
    ui->outputFilePath_lineEdit->setEnabled(!val);
    isOutputEnabled = val;
}

QString TimerControllerModule::updateTimeLabel()
{
    QString newTime = currentTime.toString(timeFormat);
    QString newPrefix = getPrefix();

    ui->time_label->setText(newTime);
    ui->prefix_label->setText(newPrefix);

    return newPrefix + newTime;
}

void TimerControllerModule::setInfoLabel(const QString &txt)
{
    ui->Info_label->setText(txt);
    ui->labelInfo_lineEdit->clear();
}

void TimerControllerModule::updateOutputFile(const QString &outputTxt) const
{
    if(!isOutputEnabled){
        return;
    }

    if(isOutputEnabled){
        QString filePath = ui->outputFilePath_lineEdit->text().trimmed();
        QFile file(filePath);
        if (!file.open(QIODevice::WriteOnly | QIODevice::Text))
            return;

        QTextStream out(&file);
        out << outputTxt;
    }
}

QString TimerControllerModule::getPrefix() const
{
    if(getIncrement() > 0){
        return tPlus;
    }else if (getIncrement() < 0){
        return tMinus;
    }
    return ui->prefix_label->text();
}

bool TimerControllerModule::isTimeExpired() const
{
    return currentTime.hour() == 0 && currentTime.minute() == 0 && currentTime.second() == 0;
}

int TimerControllerModule::getIncrement() const
{
    if(isHold){
        return 0;
    }
    return increment;
}

