#include "treeviewermodule.h"
#include "ui_treeviewermodule.h"

#include <QLabel>
#include <QDebug>
#include <QTreeWidget>
#include "Core/modulemessagesbroker.h"
#include "treevieweritem.h"

TreeViewerModule::TreeViewerModule(QWidget *parent) : DefaultModule(parent), ui(new Ui::TreeViewerModule)
{
    ui->setupUi(this);
    defaultContextMenuSetup();
    setupUI();
}

TreeViewerModule::~TreeViewerModule()
{
    delete ui;
    // The tree viewer items are deleted automatically with the ui QTreeWidget
}

void TreeViewerModule::setupUI()
{
    connect(ui->lineEdit_receivingTopic, &QLineEdit::textChanged, this, &TreeViewerModule::onLineEditTopicTextChanged);
//    XmlObject test;
//    test.fromXml("<Root>"
//                     "<Child1 name='aaaa'>"
//                         "<Child1.1/>"
//                         "<Child1.2 name='boo1.2' attr1='provac1.2'/>"
//                         "<Child1.3/>"
//                     "</Child1>"
//                     "<Child2 name='boo' attr1='provac2'/>"
//                     "<Child3/>"
//                 "</Root>"
//                 "");

    //showXmlTree(test);

}

void TreeViewerModule::showXmlTree(const XmlObject &xml)
{
    clearTree();
    TreeViewerItem* root = createTreeItemFromXml(xml);
    addItem(root);
    headers.append("Object Name");
    updateHeaders(root->getAttributesNameList());

    for(int i = 0; i < xml.childCount(); i++){
        addXmlItem(xml.childAt(i), root);
    }

    updateItemAttributesColumn();
}

void TreeViewerModule::addXmlItem(const XmlObject &xml, TreeViewerItem* parent)
{
    TreeViewerItem* item = createTreeItemFromXml(xml);
    addItem(parent, item);
    updateHeaders(item->getAttributesNameList());

    for(int i = 0; i < xml.childCount(); i++){
        addXmlItem(xml.childAt(i), item);
    }
}

TreeViewerItem *TreeViewerModule::createTreeItemFromXml(const XmlObject &xml)
{
    return new TreeViewerItem(xml);
}

void TreeViewerModule::updateHeaders(const QList<QString> itemAttributes)
{
    bool updateHeader = false;
    for(int i = 0; i < itemAttributes.count(); i++){
        if(!headers.contains(itemAttributes[i])){
            headers.append(itemAttributes[i]);
            updateHeader = true;
        }
    }

    if(updateHeader){
        tree()->setColumnCount(headers.count());
        tree()->setHeaderLabels(headers);
    }
}

void TreeViewerModule::updateItemAttributesColumn()
{
    for(TreeViewerItem* item : items){
        item->setAttributesColumn(headers);
    }
}

void TreeViewerModule::setReceivingTopic(const QString &txt)
{
    getCore()->getModuleMessagesBroker()->unsubscribe(receivingTopic, this);
    getCore()->getModuleMessagesBroker()->subscribe(txt, this, [this](const ModuleMessage &msg){
        onMsgReceived(msg);
    });

    receivingTopic = txt;
    if(ui->lineEdit_receivingTopic->text() != txt){
        ui->lineEdit_receivingTopic->setText(txt);
    }
}

void TreeViewerModule::onLineEditTopicTextChanged(const QString &txt)
{
    setReceivingTopic(txt);
}

void TreeViewerModule::clearTree()
{
    for(int i = 0; i < items.count(); i++){
        delete items[i];
    }
    items.clear();

    headers.clear();
}

QTreeWidget *TreeViewerModule::tree()
{
    return ui->treeWidget;
}

void TreeViewerModule::onMsgReceived(const ModuleMessage &msg)
{
    XmlObject xml;
    xml.fromXml(msg.payload());
    if(!xml.isEmpty()){
        showXmlTree(xml);
    }
}

QWidget *TreeViewerModule::toWidget()
{
    return this;
}

XmlObject TreeViewerModule::toXmlObject()
{
    XmlObject xml(getName(ModuleId::TREEVIEWER));

    if(receivingTopic != SkywardHubStrings::treeViewerReceivingBaseTopic){
        xml.addAttribute("ReceivingTopic", receivingTopic);
    }

    return xml;
}

void TreeViewerModule::fromXmlObject(const XmlObject &xmlObject)
{
    QString attrName = "ReceivingTopic";
    if(xmlObject.hasAttribute(attrName)){
        setReceivingTopic(xmlObject.getAttribute(attrName));
    }else{
        setReceivingTopic(SkywardHubStrings::treeViewerReceivingBaseTopic);
    }
}

void TreeViewerModule::addItem(TreeViewerItem *rootItem)
{
    tree()->addTopLevelItem(rootItem);
    //tree()->setItemWidget(rootItem, 1, new QLabel(QString::number(i)));
    items.insert(0,rootItem);
}

void TreeViewerModule::addItem(TreeViewerItem *parentItem, TreeViewerItem *childItem)
{
    if(parentItem != nullptr && childItem != nullptr){
        parentItem->addChild(childItem);
//        tree()->setItemWidget(item2, 1, new QLabel(QString::number(i)+"."+QString::number(c)));
        items.insert(0,childItem);
    }
}


