#ifndef TREEVIEWERMODULE_H
#define TREEVIEWERMODULE_H

#include "Modules/DefaultModule/defaultmodule.h"
#include "Core/xmlobject.h"
#include "Core/modulemessage.h"

class QTreeWidget;
class TreeViewerItem;

namespace Ui {
class TreeViewerModule;
}

class TreeViewerModule : public DefaultModule
{
    Q_OBJECT

public:
    explicit TreeViewerModule(QWidget *parent = nullptr);
    ~TreeViewerModule();

    QWidget *toWidget() override;

    XmlObject toXmlObject() override;
    void fromXmlObject(const XmlObject &xmlObject) override;

protected:
    void setupUI();
    QTreeWidget *tree();

    void onMsgReceived(const ModuleMessage &msg);
    void addItem(TreeViewerItem* rootItem);
    void addItem(TreeViewerItem *parentItem, TreeViewerItem* childItem);
    void showXmlTree(const XmlObject &xml);
    void addXmlItem(const XmlObject &xml, TreeViewerItem *parent);
    TreeViewerItem* createTreeItemFromXml(const XmlObject &xml);
    void updateHeaders(const QList<QString> itemAttributes);
    void updateItemAttributesColumn();
    void setReceivingTopic(const QString &txt);
    void onLineEditTopicTextChanged(const QString &txt);
    void clearTree();

private:
    Ui::TreeViewerModule *ui;
    QList<TreeViewerItem*> items;
    QList<QString> headers;
    QString receivingTopic = SkywardHubStrings::treeViewerReceivingBaseTopic;
};


#endif // TREEVIEWERMODULE_H
