#include "modulespicker.h"
#include "ui_modulespicker.h"

#include "Core/modulesmanager.h"
#include <QPushButton>
#include <QDebug>

ModulesPicker::ModulesPicker() : QDialog(), ui(new Ui::ModulesPicker)
{
    ui->setupUi(this);
    createAndConnectUI();

    this->setAttribute(Qt::WA_DeleteOnClose, true);  // WA_DeleteOnClose is set to true, so this widget will be deleted on close
}

ModulesPicker::~ModulesPicker()
{
    delete ui;
}


void ModulesPicker::createAndConnectUI()
{
    connect(ui->button_createModule, &QPushButton::clicked, this, &ModulesPicker::onCreateModuleButtonClick);

    // The splitter is handled separatelly from the others modules because it need parameters.
    // An alternative would be to remove the parameters and in the  constructor of the splitter ask to the user (with a popup)
    // wich orientation he prefere. Or just remove the choice and use the context menu to chang orientation.
    connect(ui->button_splitter,&QPushButton::clicked,this, &ModulesPicker::onSplitterButtonClick);



    /*  Important:
     *  The text of this buttons is set to be the name of the module to load onclick
     */
    for(ModuleInfo moduleInfo : ModulesManager::getModulesInfo()){


        switch (moduleInfo.getModuleCategory()) {
            case ModuleCategory::DEFAULT : ui->comboBox_modulesList->addItem(moduleInfo.getModuleName()); break;
            case ModuleCategory::DATASOURCE :
                ui->data_source_layout->insertWidget(0, createButton(moduleInfo.getModuleName()));
                break;
            case ModuleCategory::DATAVISUAL :
                ui->data_visual_layout->insertWidget(0, createButton(moduleInfo.getModuleName()));
                break;
            case ModuleCategory::UTILITY :
                ui->utility_layout->insertWidget(0, createButton(moduleInfo.getModuleName()));
                break;
            case ModuleCategory::HOME :
                ui->home_layout->insertWidget(0, createButton(moduleInfo.getModuleName()));
                break;
            default:
                break;
        }
    }

}

void ModulesPicker::connectSlot(QPushButton *button)
{
    connect(button, &QPushButton::clicked, this, [this, button](){
        Module* module = ModulesManager::instantiateModuleByName(button->text());
        if(module != nullptr){
            onModuleSelected(module);
        }
    });
}

void ModulesPicker::onCreateModuleButtonClick()
{
    QString moduleName = ui->comboBox_modulesList->currentText();
    onModuleSelected(moduleName);
}

void ModulesPicker::onSplitterButtonClick()
{
    Qt::Orientation orientation = Qt::Horizontal;
    if(ui->radioButton_vertical->isChecked()){
        orientation = Qt::Vertical;
    }

    XmlObject params;
    params.addAttribute("Orientation", orientation);


    Module* module = ModulesManager::instantiateModuleById(ModuleId::SPLITTER);
    if(module != nullptr){
        module->initialize(params);
        onModuleSelected(module);
    }
}

QPushButton *ModulesPicker::createButton(const QString &name)
{
    QPushButton *button = new QPushButton(name);
    connect(button, &QPushButton::clicked,this, [this, button](){
        if(button != nullptr){
            onModuleSelected(button->text());
        }
    });

    return button;
}


void ModulesPicker::onModuleSelected(const QString &mName)
{
    Module *module = ModulesManager::instantiateModuleByName(mName);
    if(module != nullptr)
        onModuleSelected(module);
}

void ModulesPicker::onModuleSelected(Module *module)
{
    *selectedModule = module;
    emit moduleSelected(module);
    this->close(); // WA_DeleteOnClose is set to true, so this widget will be deleted on close
}



Module *ModulesPicker::start()
{
    Module* mSelected = nullptr;
    selectedModule = &mSelected;
    this->exec();

    return mSelected;
}

//QWidget *PanelsPickerModule::toWidget()
//{
//    return this;
//}

//XmlObject PanelsPickerModule::toXmlObject()
//{

//}

//void PanelsPickerModule::fromXmlObject(const XmlObject &xmlObject)
//{
//    Q_UNUSED(xmlObject);
//}
