#include "saveconfigurationdialog.h"
#include "ui_saveconfigurationdialog.h"

#include <QFileDialog>


SaveConfigurationDialog::SaveConfigurationDialog(QWidget *parent) : QDialog(parent), ui(new Ui::SaveConfigurationDialog)
{
    ui->setupUi(this);
    this->setAttribute(Qt::WA_DeleteOnClose, true); // WA_DeleteOnClose is set to true, so this widget will be deleted on close

    connect(ui->pushButton_saveConfig, &QPushButton::clicked, this, &SaveConfigurationDialog::onSaveConfigClicked);
    connect(ui->pushButton_selectIcon, &QPushButton::clicked, this, &SaveConfigurationDialog::onSelectIconPathClicked);
}

SaveConfigurationDialog::~SaveConfigurationDialog()
{
    delete ui;
}

void SaveConfigurationDialog::pickConfigData(QString *name, QString *iconPath, QString *description)
{
    this->name = name;
    this->iconPath = iconPath;
    this->description = description;

    this->exec();
}

QString SaveConfigurationDialog::getConfigName() const
{
    return ui->textEdit_configName->text().trimmed();
}

QString SaveConfigurationDialog::getConfigIconPath() const
{
    return ui->textEdit_iconPath->toPlainText().trimmed();
}

QString SaveConfigurationDialog::getConfigDescription() const
{
    return ui->textEdit_description->toPlainText().trimmed();
}

void SaveConfigurationDialog::setConfigName(const QString &value)
{
    ui->textEdit_configName->setText(value);
}

void SaveConfigurationDialog::setConfigIconPath(const QString &value)
{
    ui->textEdit_iconPath->setText(value);
}

void SaveConfigurationDialog::setConfigDescription(const QString &value)
{
    ui->textEdit_description->setText(value);
}

void SaveConfigurationDialog::onSelectIconPathClicked()
{
    QString filePath = QFileDialog::getOpenFileName(nullptr, "Select config icon","","(*.jpeg, *.png., *.jpg)");
    setConfigIconPath(filePath);
}

void SaveConfigurationDialog::onSaveConfigClicked()
{
    (*name) = getConfigName();
    (*iconPath) = getConfigIconPath();
    (*description) = getConfigDescription();
    this->close();
}
