#include "subscriptionspanel.h"
#include "ui_subscriptionspanel.h"

#include <QDebug>

SubscriptionsPanel::SubscriptionsPanel(const QList<QString> *subscriptionsList, QWidget *parent) : QWidget(parent), ui(new Ui::SubscriptionsPanel)
{
    ui->setupUi(this);
    this->setAttribute(Qt::WA_DeleteOnClose, true);

    for(auto s : *subscriptionsList){
        addSubscription(s);
    }

    connect(ui->button_addTopic, &QPushButton::clicked, this, &SubscriptionsPanel::onAddSubscriptionClicked);
    connect(ui->button_remove, &QPushButton::clicked, this, &SubscriptionsPanel::onRemoveClicked);
    connect(ui->lineEdit_topic, &QLineEdit::returnPressed, this, &SubscriptionsPanel::onAddSubscriptionClicked);
    connect(ui->subscriptions_list, &QListWidget::itemDoubleClicked, this, &SubscriptionsPanel::onItemDoubleClick);
}

SubscriptionsPanel::~SubscriptionsPanel()
{
    delete ui;
}

void SubscriptionsPanel::addSubscription(const QString &s)
{
    QListWidgetItem *listItem = new QListWidgetItem(s);
    //listItem->setCheckState(Qt::Checked);
    //listItem->setFlags(listItem->flags() | Qt::ItemIsEditable);
    ui->subscriptions_list->addItem(listItem);
    emit SubscriptionAdded(s);
}

void SubscriptionsPanel::removeSubscription(QListWidgetItem *item)
{
    emit SubscriptionRemoved(item->text());
    delete item;
}


void SubscriptionsPanel::onAddSubscriptionClicked()
{
    QString newSubscription = ui->lineEdit_topic->text().trimmed();
    if(newSubscription.length()>0){
        addSubscription(newSubscription);
    }
}

void SubscriptionsPanel::onRemoveClicked()
{
    auto items = ui->subscriptions_list->selectedItems();

    for(auto item : items){
        removeSubscription(item);
    }
}

void SubscriptionsPanel::onItemDoubleClick(QListWidgetItem *item)
{
    ui->lineEdit_topic->setText(item->text());
}








