#include "valueelement.h"

#include <QRadioButton>

#include <QDebug>

ValueElement::ValueElement()
{

}

ValueElement::~ValueElement()
{

}

ValueElement::ValueElement(const ValueElement &other)
{
    copy(other);
}

ValueElement::ValueElement(const QString &name, const QString &topic, const QString &receivedValue, const QString &displayedValue, const QString &color, const QString &outputTopic)
{
    setName(name);
    setTopic(topic);
    setReceivedValue(receivedValue);
    setDisplayedValue(displayedValue);
    setColor(color);
    setOutpuTopic(outputTopic);
}

ValueElement ValueElement::operator=(const ValueElement &other)
{
    copy(other);
    return *this;
}

QString ValueElement::getName() const
{
    return name;
}

void ValueElement::setName(const QString &value)
{
    name = value;
}

QString ValueElement::getTopic() const
{
    return topic;
}

void ValueElement::setTopic(const QString &value)
{
    topic = value;
}

QString ValueElement::getReceivedValue() const
{
    return receivedValue;
}

void ValueElement::setReceivedValue(const QString &value)
{
    receivedValue = value;
}

QString ValueElement::getDisplayedValue() const
{
    return displayedValue;
}

void ValueElement::setDisplayedValue(const QString &value)
{
    displayedValue = value;
}

QString ValueElement::getColor() const
{
    return color;
}

void ValueElement::setColor(const QString &value)
{
    color = value;
}

QString ValueElement::toString() const
{
    return getName() + ": " + getTopic() + " [" + getReceivedValue() + " = " + getDisplayedValue() + "] color=" + getColor();
}

XmlObject ValueElement::toXmlObject() const
{
    XmlObject obj("Rule");
    obj.addAttribute("name", getName());
    obj.addAttribute("topic", getTopic());
    obj.addAttribute("received", getReceivedValue());
    obj.addAttribute("displayed", getDisplayedValue());
    obj.addAttribute("color", getColor());
    obj.addAttribute("outputTopic", getOutpuTopic());
    obj.addAttribute("displayInView", isDisplayInView());
    return obj;
}

bool ValueElement::fromXmlObject(const XmlObject &xmlObject)
{
    bool result = false;
    if(xmlObject.getObjectName() == "Rule" && xmlObject.hasAttribute("name")){
        QString name = xmlObject.getAttribute("name");
        if(name != ""){
            setName(name);
            result = true;
        }

        setTopic(xmlObject.getAttribute("topic"));
        setReceivedValue(xmlObject.getAttribute("received"));
        setDisplayedValue(xmlObject.getAttribute("displayed"));
        setColor(xmlObject.getAttribute("color"));
        setOutpuTopic(xmlObject.getAttribute("outputTopic"));

        int display = 1;
        if(xmlObject.getIntAttribute("displayInView", display) && (display == 0 || display == 1)){
            setDisplayInView(display);
        }
    }
    return result;
}

void ValueElement::copy(const ValueElement &other)
{
    setName(other.getName());
    setTopic(other.getTopic());
    setReceivedValue(other.getReceivedValue());
    setDisplayedValue(other.getDisplayedValue());
    setColor(other.getColor());
    setOutpuTopic(other.getOutpuTopic());
    setDisplayInView(other.isDisplayInView());
}

bool ValueElement::isDisplayInView() const
{
    return displayInView;
}

void ValueElement::setDisplayInView(bool value)
{
    displayInView = value;
}

QString ValueElement::getOutpuTopic() const
{
    return outpuTopic;
}

void ValueElement::setOutpuTopic(const QString &value)
{
    outpuTopic = value;
}

bool ValueElement::isEmitActive() const
{
    return outpuTopic != "";
}

QString ValueElement::getCurrentValue() const
{
    return currentValue;
}

bool ValueElement::updateCurrentValue(const QString &value)
{
    if(getReceivedValue() == "" || value == getReceivedValue()){
        if(getDisplayedValue() != ""){
            currentValue = getDisplayedValue();
        }
        else{
            currentValue = value;
        }
        return true;
    }
    return false;
}

