#ifndef MODULEINFO_H
#define MODULEINFO_H

#include <QStringList>

class Module;

enum ModuleId{
    EMPTY,
    SPLITTER,
    SKYWARDHUB,
    COMMANDPAD,
    TELEMETRYPAD,
    BROKERTEST,
    GRAPH,
    TABLE,
    IMAGEVIEWER,
    MESSAGEVIEWER,
    MAVLINK,
    FSMVIEWER,
    TREEVIEWER,
    SCROLLAREA,
    FILESTREAM,
    STATEVIEWER,
    VALUESCONVERTERVIEWER,
    MAVLINK_RCK_TESTING,
    TIMER_CONTROLLER,
    MODULETEST
};

enum ModuleCategory{
    HIDDEN,
    DEFAULT,
    UTILITY,
    DATAVISUAL,
    DATASOURCE,
    HOME
};

class ModuleInfo
{
public:
    ModuleInfo();
    ModuleInfo(const ModuleInfo &other);
    ModuleInfo(const ModuleId &id, const QString &name);
    ModuleInfo(const ModuleId &id, const QString &name, ModuleCategory category);

    ModuleId getId() const;
    void setId(const ModuleId &value);

    QString getModuleName() const;
    void setModuleName(const QString &value);

    std::function<Module *()> getFactory() const;
    void setFactory(const std::function<Module *()> &value);

    QStringList getModuleSourceFiles() const;
    void addModuleSourceFiles(const QString &value);

    void operator=(const ModuleInfo &other);

    ModuleCategory getModuleCategory() const;
    void setCategory(const ModuleCategory &value);
protected:
    void copy(const ModuleInfo &other);

private:
    ModuleId moduleId;
    ModuleCategory mCategory = ModuleCategory::DEFAULT;
    QString moduleName;
    std::function<Module *()> factory;
    QStringList moduleSourceFiles;
};

#endif // MODULEINFO_H
