#include "moduleslist.h"
#
// ____________________________________________________________________________________________

#include "Modules/Empty/emptymodule.h"
#include "Modules/Utility/modulespicker.h"
#include "Modules/Splitter/splittermodule.h"
#include "Modules/SkywardHub/skywardhubmodule.h"
#include "Modules/CommandPad/commandpadmodule.h"
#include "Modules/CommandPad/telemetryrequestmodule.h"
#include "Modules/Test/testmodule.h"
#include "Modules/Graph/graphmodule.h"
#include "Modules/Table/tablemodule.h"
#include "Modules/ImageViewer/imageviewermodule.h"
#include "Modules/MessageViewer/messagesviewermodule.h"
#include "Modules/Mavlink/mavlinkmodule.h"
#include "Modules/TreeViewer/treeviewermodule.h"
#include "Modules/ScrollArea/scrollareamodule.h"
#include "Modules/FileStream/filestreammodule.h"
#include "Modules/Mavlink/mavlinkrocketmsgtestingmodule.h"
#include "Modules/StateViewer/stateviewermodule.h"
#include "Modules/ValuesConverterViewer/valuesconverterviewermodule.h"
#include "Modules/TimerController/timercontrollermodule.h"

void ModulesList::createModuleList()
{
    /*
     *  addModule(MODULE UNIQUE ID, MODULE UNIQUE NAME, a function that instantiate your module)
     *  If you are creating a new Module, please add it's id in the ModuleId enum (you can find it in moduleslist.h)
     *
     *
     * WARNING!
     * ⚠ Each ID must be unique
     * ⚠ Each ModuleName must be unique and WITHOUT spaces
     * ⚠ A Module must NOT include a file outside its folder
     * ⚠ The ModuleSourceFiles allow you to specify which files will be included
     *       (all the files that match the strings you privide will be included)
     */

    #ifdef EMPTYMODULE_H
    ModuleInfo empty(ModuleId::EMPTY, "EmptyModule");
    empty.setFactory([](){return new EmptyModule();});
    empty.addModuleSourceFiles("Modules/Empty/");
    addModuleInfo(empty);
    #endif

    #ifdef SPLITTERMODULE_H
    ModuleInfo splitter(ModuleId::SPLITTER, "Splitter");
    splitter.setFactory([](){return new SplitterModule();});
    splitter.addModuleSourceFiles("Modules/Splitter/");
    addModuleInfo(splitter);
    #endif

    #ifdef SKYWARDHUBMODULE_H
    ModuleInfo hub(ModuleId::SKYWARDHUB, "SkywardHub", ModuleCategory::HOME);
    hub.setFactory([](){return new SkywardHubModule();});
    hub.addModuleSourceFiles("Modules/SkywardHub/");
    addModuleInfo(hub);
    #endif

    #ifdef COMMANDPADMODULE_H
    ModuleInfo cmdPad(ModuleId::COMMANDPAD, "CommandPad", ModuleCategory::UTILITY);
    cmdPad.setFactory([](){return new CommandPadModule();});
    cmdPad.addModuleSourceFiles("Modules/CommandPad/");
    addModuleInfo(cmdPad);
    #endif

    #ifdef TELEMETRYREQUESTMODULE_H
    ModuleInfo telemetryPad(ModuleId::TELEMETRYPAD, "TelemetryPad", ModuleCategory::UTILITY);
    telemetryPad.setFactory([](){return new TelemetryRequestModule();});
    telemetryPad.addModuleSourceFiles("Modules/CommandPad/");
    addModuleInfo(telemetryPad);
    #endif

    #ifdef TESTMODULE_H
    ModuleInfo testModule(ModuleId::BROKERTEST, "TestBroker", ModuleCategory::DEFAULT);
    testModule.setFactory([](){return new TestModule();});
    testModule.addModuleSourceFiles("Modules/Test/");
    addModuleInfo(testModule);
    #endif

    #ifdef GRAPHMODULE_H
    ModuleInfo graphModule(ModuleId::GRAPH, "Graph", ModuleCategory::DATAVISUAL);
    graphModule.setFactory([](){return new GraphModule();});
    graphModule.addModuleSourceFiles("Modules/Graph/");
    addModuleInfo(graphModule);
    #endif

    #ifdef TABLEMODULE_H
    ModuleInfo tableModule(ModuleId::TABLE, "Table", ModuleCategory::DATAVISUAL);
    tableModule.setFactory([](){return new TableModule();});
    tableModule.addModuleSourceFiles("Modules/Table/");
    addModuleInfo(tableModule);
    #endif

    #ifdef IMAGEVIEWERMODULE_H
    ModuleInfo imageViewer(ModuleId::IMAGEVIEWER, "ImageViewer", ModuleCategory::DATAVISUAL);
    imageViewer.setFactory([](){return new ImageViewerModule();});
    imageViewer.addModuleSourceFiles("Modules/ImageViewer/");
    addModuleInfo(imageViewer);
    #endif

    #ifdef MESSAGESVIEWERMODULE_H
    ModuleInfo msgViewer(ModuleId::MESSAGEVIEWER, "MessageViewer", ModuleCategory::DATAVISUAL);
    msgViewer.setFactory([](){return new MessagesViewerModule();});
    msgViewer.addModuleSourceFiles("Modules/MessageViewer/");
    addModuleInfo(msgViewer);
    #endif

    #ifdef MAVLINKMODULE_H
    ModuleInfo mavlink(ModuleId::MAVLINK, "Mavlink", ModuleCategory::DATASOURCE);
    mavlink.setFactory([](){return new MavlinkModule();});
    mavlink.addModuleSourceFiles("Modules/Mavlink/");
    addModuleInfo(mavlink);
    #endif

    #ifdef MAVLINKROCKETMSGTESTINGMODULE_H
    ModuleInfo mvkRckTesting(ModuleId::MAVLINK_RCK_TESTING, "MavlinkRocketMsgTesting", ModuleCategory::DEFAULT);
    mvkRckTesting.setFactory([](){return new MavlinkRocketMsgTestingModule();});
    addModuleInfo(mvkRckTesting);
    #endif

    #ifdef TREEVIEWERMODULE_H
    ModuleInfo treeViewer(ModuleId::TREEVIEWER, "TreeViewer", ModuleCategory::UTILITY);
    treeViewer.setFactory([](){return new TreeViewerModule();});
    treeViewer.addModuleSourceFiles("Modules/TreeViewer/");
    addModuleInfo(treeViewer);
    #endif

    #ifdef SCROLLAREAMODULE_H
    ModuleInfo scrollArea(ModuleId::SCROLLAREA, "ScrollArea", ModuleCategory::UTILITY);
    scrollArea.setFactory([](){return new ScrollAreaModule();});
    scrollArea.addModuleSourceFiles("Modules/ScrollArea/");
    addModuleInfo(scrollArea);
    #endif

    #ifdef FILESTREAMMODULE_H
    ModuleInfo fileStream(ModuleId::FILESTREAM, "FileStream", ModuleCategory::DATASOURCE);
    fileStream.setFactory([](){return new FileStreamModule();});
    fileStream.addModuleSourceFiles("Modules/FileStream/");
    addModuleInfo(fileStream);
    #endif

    #ifdef STATEVIEWERMODULE_H
    ModuleInfo stateViewer(ModuleId::STATEVIEWER, "StateViewer", ModuleCategory::DATAVISUAL);
    stateViewer.setFactory([](){return new StateViewerModule();});
    stateViewer.addModuleSourceFiles("Modules/StateViewer/");
    addModuleInfo(stateViewer);
    #endif

    #ifdef VALUESCONVERTERVIEWERMODULE_H
    ModuleInfo valuesconverterviewer(ModuleId::VALUESCONVERTERVIEWER, "ValuesConverterViewer", ModuleCategory::UTILITY);
    valuesconverterviewer.setFactory([](){return new ValuesConverterViewerModule();});
    valuesconverterviewer.addModuleSourceFiles("Modules/ValuesConverterViewer/");
    addModuleInfo(valuesconverterviewer);
    #endif

    #ifdef TIMERCONTROLLERMODULE_H
    ModuleInfo timerController(ModuleId::TIMER_CONTROLLER, "TimerController", ModuleCategory::UTILITY);
    timerController.setFactory([](){return new TimerControllerModule();});
    timerController.addModuleSourceFiles("Modules/TimerController/");
    addModuleInfo(timerController);
    #endif
}


// ____________________________________________________________________________________________

ModulesList::ModulesList()
{
    createModuleList();
}

bool ModulesList::containsId(ModuleId id)
{    
    for(int i = 0; i< modulesInfo.count(); i++){
        if(modulesInfo[i].getId() == id)
            return true;
    }
    return false;
}

void ModulesList::addModuleInfo(const ModuleInfo &info)
{
    modulesInfo.append(info);
}

QString ModulesList::getModuleName(ModuleId id) const
{
    for(int i = 0; i< modulesInfo.count(); i++){
        if(modulesInfo[i].getId() == id)
            return modulesInfo[i].getModuleName();
    }
    return "";
}

std::function<Module *()> ModulesList::getFactory(ModuleId id) const
{
    for(int i = 0; i< modulesInfo.count(); i++){
        if(modulesInfo[i].getId() == id)
            return modulesInfo[i].getFactory();
    }
    return [](){return nullptr;};
}

std::function<Module *()> ModulesList::findFactoryByModuleName(const QString &name)
{
    for(int i = 0; i< modulesInfo.count(); i++){
        if(modulesInfo[i].getModuleName() == name)
            return modulesInfo[i].getFactory();
    }
    return [](){return nullptr;};
}

QList<QString> ModulesList::getModulesNamesList()
{
    QList<QString> names;
    for(int i = 0; i< modulesInfo.count(); i++){
        names.append(modulesInfo[i].getModuleName());
    }
    return names;
}

QList<ModuleInfo> ModulesList::getModulesInfo()
{
    return modulesInfo;
}
